#!/usr/bin/env python3
#
# Author: Rajendra Kumar
#
# This file is part of gcMapExplorer
# Copyright (C) 2016-2017  Rajendra Kumar, Ludvig Lizana, Per Stenberg
#
# gcMapExplorer is a free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gcMapExplorer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gcMapExplorer.  If not, see <http://www.gnu.org/licenses/>.
#
#=============================================================================

import numpy as np
import tempfile
import scipy.sparse as sparse
# import scipy.sparse.linalg as sparse_linalg
from scipy.linalg import get_blas_funcs
from scipy import linalg as sp_linalg
import scipy.stats as stats
import string, random
import os, re

from scipy.stats import mstats
import numpy.ma as ma

dtype_npBINarray = 'float32'


def sorted_nicely( l ):
	""" Sorts the given iterable in the way that is expected.

	Required arguments:
	l -- The iterable to be sorted.

	"""
	convert = lambda text: int(text) if text.isdigit() else text
	alphanum_key = lambda key: [convert(c) for c in re.split('([0-9]+)', key)]
	return sorted(l, key = alphanum_key)

def locate_significant_digit_after_decimal(value):
	""" Get location at which significant digit start after decimal
	"""
	location = 0
	base = 0
	while( base <=0 ):
		base = int( value * 10 ** location  )
		location = location + 1

	return location

def kth_diag_indices(k, a):
	""" Get diagonal indices of array 'a' offset by 'k'
	"""
	rows, cols = np.diag_indices_from(a)
	if k < 0:
		return rows[:k], cols[-k:]
	elif k > 0:
		return rows[k:], cols[:-k]
	else:
		return rows, cols

# Calling blas dot function for matrix
def mdot(v, w):
	gemm = get_blas_funcs("gemm", [v, w])
	return gemm(alpha=1., a=v, b=w)

class MemoryMappedArray:
	"""Convenient wrapper for numpy memory mapped array file

	For more details, see here: (See: |numpy memmap|).

	Attributes
	----------
	path2matrix : str
		Path to numpy memory mapped array file
	arr : numpy.memmap
		Pointer to memory mapped numpy array
	workDir : str
		Path to the directory where temporary intermediate files are generated. If ``None``, files are generated in the temporary directory according to the OS type.
	dtype : str
		Data type of array

	Parameters
	----------
	shape : tuple
		Shape of array
	fill : int or float (Optional)
		Fill array with this value
	dtype : str
		Data type of array

	"""
	def __init__(self, shape, fill=None, workDir=None, dtype='float32'):
		self.workDir = workDir

		# Generating a file with random name
		(fd, fname) = tempfile.mkstemp(suffix='.tmp', prefix='nparray_', dir=self.workDir, text=False)
		os.close(fd)     # Close file, error in windows OS

		# Generating a memmap array
		self.path2matrix = fname
		self.arr = np.memmap(self.path2matrix, dtype=dtype, mode='w+', shape=shape)
		self.dtype = dtype
		if fill is not None:
			self.arr.fill(fill)

	def copy(self):
		"""Copy this numpy memory mapped array and generate new

		Returns
		-------
		out : :class:`MemoryMappedArray`
			A new :class:`MemoryMappedArray` instance with copied arrays

		"""
		out = MemoryMappedArray(self.arr.shape, workDir=self.workdir, dtype=self.dtype)
		out.arr[:] = self.arr[:]
		return out

	def copy_from(self, src):
		"""Copy values from source :class:`MemoryMappedArray`

		Parameters
		----------
		src : :class:`MemoryMappedArray`
			Source memory mapped arrays for new values

		Returns
		-------
		None


		Raises
		------
		ValueError
			if src is not of :class:`MemoryMappedArray` instance

		"""
		if not isinstance(src, MemoryMappedArray):
			raise ValueError ('Not a MemoryMappedArray !!')

		self.arr[:] = src.arr[:]
		self.arr.flush()

	def copy_to(self, dest):
		"""Copy values to destination :class:`MemoryMappedArray`

		Parameters
		----------
		dest : :class:`MemoryMappedArray`
			Destination memory mapped arrays

		Returns
		-------
		None


		Raises
		------
		ValueError
			if dest is not of :class:`MemoryMappedArray` instance

		"""

		if type(dest) is not MemoryMappedArray:
			raise ValueError ('Not a MemoryMappedArray !!')
		dest.arr[:] = self.arr[:]
		#dest.arr.flush()

	def __del__(self):
		del self.arr
		try:
			os.remove(self.path2matrix)
		except:
			pass

def get_nonzeros_index(matrix, thershold_percentile=None, thershold_data_occup=None):
	"""To get a numpy array of bool values for all rows/columns which have **NO** missing data

	Parameters
	----------
	matrix : numpy.memmap or :attr:`gcMapExplorer.lib.ccmap.CCMAP.matrix`
		Input matrix
	percentile_thershold_no_data : int
		It can be used to filter the map, where rows/columns with largest numbers of missing data can be discarded.
		``percentile_thershold_no_data`` should be between 1 and 100. This options discard the rows and columns which are above this percentile.
		For example: if this value is 99, those row or columns will be discarded which contains larger than number of zeros (missing data) at 99 percentile.

		To calculate percentile, all blank rows are removed, then in all rows, number of zeros are counted. Afterwards, number of zeros at
		`percentile_thershold_no_data` percentile is obtained. In next step, if a row contain number of zeros larger than this percentile value,
		the whole row and column is assigned to have missing data. This percentile indicates highest numbers of zeros (missing data) in given rows/columns.

	thershold_data_occup : float
		It can be used to filter the map, where rows/columns with largest numbers of missing data can be discarded.
		This ratio is (number of bins with data) / (total number of bins in the given row/column).
		For example: if `thershold_data_occup = 0.8`, then all rows containing more than 20\% of missing data will be discarded.

		Note that this parameter is suitable for low resolution data because maps are likely to be much less sparse.

	Returns
	-------
	bData : numpy.array[bool]
		1D-array containing ``True`` and ``False`` values.
		* If ``True``: row/column has data above the thershold
		* If ``False``: row/column has no data under the thershold
	"""
	bData = np.empty(shape=matrix.shape[0], dtype='bool')
	mx = matrix.shape[0]
	#my = matrix.shape[1]

	zero_count = []
	for i in range(mx):
		if np.sum( matrix[i] ) == 0.0:
			bData[i] = False
			# Although whole column/row is zero, count of zero taken as zero
			zero_count.append(0)
		else:
			bData[i] = True
			zero_count.append(np.nonzero( matrix[i] == 0 )[0].shape[0])


	zero_count = np.asarray(zero_count)

	if thershold_percentile is not None and thershold_data_occup is not None:
		raise AssertionError("Both 'thershold_percentile' and 'thershold_count_ratio' cannot be used simultaneously!")

	# Make false if number of zeros is less than thershold
	if thershold_percentile is not None:
		percentile = np.percentile(zero_count[np.nonzero(zero_count != 0)], thershold_percentile)
		for i in range(mx):
			if bData[i]:
				if zero_count[i] >= percentile:
					bData[i] = False
					#print(i, zero_count[i], percentile)

	# Make false if number of zeros is less than thershold
	if thershold_data_occup is not None:
		for i in range(mx):
			if bData[i]:
				if zero_count[i]/mx >=  (1.0 - thershold_data_occup):
					bData[i] = False
					#print(i, count[i], real_data, thershold_percentile)

	bData = np.asarray(bData)

	length = np.nonzero( bData == True )[0].shape[0]
	if length == 0:
		if thershold_percentile is not None:
			raise ValueError("All rows contain zero values!!! Try increasing 'thershold_percentile' value or do not use it.")
		if thershold_data_occup is not None:
			raise ValueError("All rows contain zero values!!! Try decreasing 'thershold_data_occup' value or do not use it.")

	return bData

def remove_zeros(matrix, thershold_percentile=None, thershold_data_occup=None, workDir=None):
	"""To remove rows/columns with missing data (zero values)

	Parameters
	----------
	matrix : numpy.memmap or :attr:`gcMapExplorer.lib.ccmap.CCMAP.matrix`
		Input matrix
	percentile_thershold_no_data : int
		It can be used to filter the map, where rows/columns with largest numbers of missing data can be discarded.
		``percentile_thershold_no_data`` should be between 1 and 100. This options discard the rows and columns which are above this percentile.
		For example: if this value is 99, those row or columns will be discarded which contains larger than number of zeros (missing data) at 99 percentile.

		To calculate percentile, all blank rows are removed, then in all rows, number of zeros are counted. Afterwards, number of zeros at
		`percentile_thershold_no_data` percentile is obtained. In next step, if a row contain number of zeros larger than this percentile value,
		the whole row and column is assigned to have missing data. This percentile indicates highest numbers of zeros (missing data) in given rows/columns.

	thershold_data_occup : float
		It can be used to filter the map, where rows/columns with largest numbers of missing data can be discarded.
		This ratio is (number of bins with data) / (total number of bins in the given row/column).
		For example: if `thershold_data_occup = 0.8`, then all rows containing more than 20\% of missing data will be discarded.

		Note that this parameter is suitable for low resolution data because maps are likely to be much less sparse.

	workDir : str
		Path to the directory where temporary intermediate files are generated. If ``None``, files are generated in the temporary directory according to the OS type.


	Returns
	-------
	A : :class:`MemoryMappedArray`
		:class:`MemoryMappedArray` instance containing new truncated array as memory mapped file
	bNoData : numpy.array[bool]
		1D-array containing ``True`` and ``False`` values.
		* If ``True``: row/column has no data under the thershold
		* If ``False``: row/column has data above the thershold

	"""

	bData = np.empty(shape=matrix.shape[0], dtype='bool')
	mx = matrix.shape[0]
	my = matrix.shape[1]

	bData = get_nonzeros_index(matrix, thershold_percentile=thershold_percentile, thershold_data_occup=thershold_data_occup)

	length = np.nonzero( bData == True )[0].shape[0]
	A = MemoryMappedArray((length, length),  workDir=workDir, dtype='float64')

	ci = -1
	cj = 0
	for i in range(mx):
		if not bData[i]:
			continue
		else:
			ci += 1
		cj = 0
		for j in range(my):
			if bData[j]:
				A.arr[ci][cj] = matrix[i][j]
				cj += 1

	return A, ~bData

def genFullMapFromTruncMap(inMatrix, outMatrix, bNoData, minvalue=None):

	if minvalue is None:
		minvalue = np.amin(inMatrix)

	inI = -1
	inJ = 0
	ox = outMatrix.shape[0]
	oy = outMatrix.shape[1]

	for outI in range(ox):
		if not bNoData[outI]:
			inI += 1

		inJ = 0
		for outJ in range(oy):
			if bNoData[outI] or bNoData[outJ]:
				outMatrix[outI][outJ] = minvalue
			else:
				if inMatrix[inI][inJ] == 0:
					outMatrix[outI][outJ] = minvalue
				else:
					outMatrix[outI][outJ] = inMatrix[inI][inJ]
				inJ += 1



def CalculateTransitionProbablity(A, bNoData, Out):
	mshape = A.shape

	minval = 9e10
	maxval = -9e10
	for m in range(mshape[0]):
		if bNoData[m]:
			continue
		sumr = A[m].sum()
		Out[m] = (A[m] / sumr)[:]

		x = np.sort(Out[m,np.nonzero(Out[m])])[0]
		minval = min(minval, x[0])
		maxval = max(maxval, x[-1])
		x=None

	for m in range(mshape[0]):
		if bNoData[m]:
			Out[m].fill(minval)
			Out[:,m].fill(minval)
		else:
			idx = np.nonzero( Out[m] == 0 )
			Out[m][idx] = minval

	return float(minval), float(maxval)

def MatrixMultiplyHDD(A, B, Out=None):
	bOut = False
	if type(A) is not MemoryMappedArray:
		raise ValueError ('Not a MemoryMappedArray !!')
	if type(B) is not MemoryMappedArray:
		raise ValueError ('Not a MemoryMappedArray !!')

	if len(A.arr.shape) == 1:
		Am = 1
		An = A.arr.shape[0]
	elif len(A.arr.shape) == 2:
		Am = A.arr.shape[0]
		An = A.arr.shape[1]
	else:
		raise ValueError ('input A must be a vector or square array')

	if len(B.arr.shape) == 1:
		Bm = B.arr.shape[0]
		Bn = 1
	elif len(B.arr.shape) == 2:
		Bm = B.arr.shape[0]
		Bn = B.arr.shape[1]
	else:
		raise ValueError ('input B must be a vector or square array')

	if An != Bm:
		raise ArithmeticError ('Multiplication not possible !!!')

	dtype='float64'

	dtypes=[A.dtype, B.dtype]

	if 'int' in str(A.dtype) and 'int' in str(B.dtype):
		dtype=sorted(dtypes)[-1]

	if 'float' in str(A.dtype) or 'float' in str(B.dtype):
		if 'float' in dtypes:
			dtype = 'float'
		if 'float32' in dtypes:
			dtype = 'float32'
		if 'float64' in dtypes:
			dtype = 'float64'


	if Out is None:
		if Am == 1 and Bn == 1:
			Out = MemoryMappedArray((1, ), dtype=dtype)
		elif Am == 1 and Bn > 1:
			Out = MemoryMappedArray((Bn, ), dtype=dtype)
		elif Am > 1 and Bn == 1:
			Out = MemoryMappedArray((Am, ), dtype=dtype)
		else:
			Out = MemoryMappedArray((Am, Bn), dtype=dtype)
	else:
		bOut = True

	if Am == 1 and Bn == 1:
		Out.arr[0] = np.dot(A.arr, B.arr)
	elif Am == 1 and Bn > 1:
		for j in range(Bn):
			Out.arr[j] = np.dot(A.arr, B.arr[:,j])
	elif Am > 1 and Bn == 1:
		for i in range(Am):
			Out.arr[i] = np.dot(A.arr[i], B.arr)
	else:
		for i in range(Am):
			for j in range(Bn):
				Out.arr[i][j] = np.dot(A.arr[i], B.arr[:,j])

	if bOut:
		return 0
	else:
		if len(Out.arr.shape) == 1 and Out.arr.shape[0] == 1:
			return Out
		else:
			return Out

def MatrixPowerHDD(M, n, result=None):
	if type(M) is not MemoryMappedArray:
		raise ValueError ('Not a MemoryMappedArray !!')

	if len(M.arr.shape) != 2 or M.arr.shape[0] != M.arr.shape[1]:
		raise ValueError("input must be a square array")

	if not np.issubdtype(type(n), int):
		raise TypeError("exponent must be an integer")

	if result is None:
		result = M.copy()
	tmp_result = M.copy()

	if n <= 3:
		for _ in range(n-1):
			MatrixMultiplyHDD(result, M, Out=tmp_result)
			result.copy_from(tmp_result)
		del tmp_result
		return result

    # binary decomposition to reduce the number of Matrix
    # multiplications for n > 3.
	beta = np.binary_repr(n)
	Z = M.copy()
	Zout = M.copy()
	q = 0
	t = len(beta)
	while beta[t-q-1] == '0':
		MatrixMultiplyHDD(Z, Z, Out=Zout)
		Z.copy_from(Zout)
		q += 1

	result.copy_from(Z)
	for k in range(q+1, t):
		MatrixMultiplyHDD(Z, Z, Out=Zout)
		Z.copy_from(Zout)
		if beta[t-k-1] == '1':
			MatrixMultiplyHDD(result, Z, Out=tmp_result)
			result.copy_from(tmp_result)

	del tmp_result
	del Z
	del Zout

	return result

def MatrixPowerRAM(M, n):
	"""Raise a square matrix to the (integer) power `n`.

	.. note::	Ported from numpy/matrixlib/defmatrix.py. This function was rewritten to speed-up the calculation. In pace of numpy
	dot function, a mdot function is used, which directly uses blas dgemm function.


	For positive integers `n`, the power is computed by repeated matrix
	squarings and matrix multiplications. If ``n == 0``, the identity matrix
	of the same shape as M is returned. If ``n < 0``, the inverse
	is computed and then raised to the ``abs(n)``.
	Parameters
	----------
	M : ndarray or matrix object
	    Matrix to be "powered."  Must be square, i.e. ``M.shape == (m, m)``,
	    with `m` a positive integer.
	n : int
	    The exponent can be any integer or long integer, positive,
	    negative, or zero.
	Returns
	-------
	M**n : ndarray or matrix object
	    The return value is the same shape and type as `M`;
	    if the exponent is positive or zero then the type of the
	    elements is the same as those of `M`. If the exponent is
	    negative the elements are floating-point.
	Raises
	------
	LinAlgError
	    If the matrix is not numerically invertible.
	See Also
	--------
	matrix
	    Provides an equivalent function as the exponentiation operator
	    (``**``, not ``^``).
	Examples
	--------
	"""

	M = np.asanyarray(M)

	if len(M.shape) != 2 or M.shape[0] != M.shape[1]:
		raise ValueError("input must be a square array")
	if not np.issubdtype(type(n), int):
		raise TypeError("exponent must be an integer")

	from numpy.linalg import inv

	if n==0:
		M = M.copy()
		M[:] = np.identity(M.shape[0])
		return M
	elif n<0:
		M = inv(M)
		n *= -1

	result = M
	if n <= 3:
		for _ in range(n-1):
			result = mdot(result, M)
		return result

	# binary decomposition to reduce the number of Matrix
	# multiplications for n > 3.
	beta = np.binary_repr(n)
	Z, q, t = M, 0, len(beta)
	while beta[t-q-1] == '0':
		Z = mdot(Z, Z)
		q += 1
	result = Z
	for k in range(q+1, t):
		Z = mdot(Z, Z)
		if beta[t-k-1] == '1':
			result = mdot(result, Z)
	return result

def CalcStationaryDistributionRAM(orig_matrix, shape, bNonZeros, stop_tol=1e-12, iteration=None):
	""" Calculate stationary distibution
	Hellloooo

	"""
	matrix = (orig_matrix[bNonZeros,:])[:,bNonZeros]   # Selected row-column which are not all zeros
	mshape = matrix.shape

	A = MemoryMappedArray(mshape, dtype='float64')
	B = MemoryMappedArray(mshape, dtype='float64')
	C = MemoryMappedArray(mshape, dtype='float64')

	A.arr[:] = matrix[:]
	B.arr[:] = matrix[:]
	C.arr[:] = matrix[:]

	if iteration is None:
		iteration = int(shape[0]/2)

	tol = 100
	prev_tol = 100
	c = 1
	while (tol >= stop_tol):
		print('Before Iteration: {0}, Tol: {1}' .format(c, tol))
		tol = LoopStaionaryDistributionRAM(matrix, A, B, C, mshape, c, 60)
		c += 60
		print('After Iteration: {0}, Tol: {1}' .format(c, tol))

		if prev_tol == tol:
			break

		if c >= iteration:
			break

		prev_tol = tol

	vector = np.zeros((shape[0],))
	v_j = 0
	vecmin = np.amin(B.arr)
	for i in range(shape[0]):
		if bNonZeros[i]:
			vector[i] = B.arr[0][v_j]
			v_j = v_j + 1
		else:
			vector[i] = vecmin

	del A
	del B
	del C

	return vector

def CalcStationaryDistributionHDD(orig_matrix, shape, bNonZeros, stop_tol=1e-12, iteration=None):

	matrix = (orig_matrix[bNonZeros,:])[:,bNonZeros]   # Selected row-column which are not all zeros
	mshape = matrix.shape

	MA = MemoryMappedArray(mshape, dtype='float64')
	A = MemoryMappedArray(mshape, dtype='float64')
	B = MemoryMappedArray(mshape, dtype='float64')
	C = MemoryMappedArray(mshape, dtype='float64')

	MA.arr[:] = matrix[:]
	A.arr[:] = matrix[:]
	B.arr[:] = matrix[:]
	C.arr[:] = matrix[:]

	if iteration is None:
		iteration = int(shape[0]/2)

	tol = 100
	prev_tol = 100
	c = 1
	while (tol >= stop_tol):
		print('Before Iteration: {0}, Tol: {1}' .format(c, tol))
		tol = LoopStaionaryDistributionHDD(MA, A, B, C, mshape, c, 10)
		c += 10
		print('After Iteration: {0}, Tol: {1}' .format(c, tol))
		if prev_tol == tol:
			break

		if c >= iteration:
			break

		prev_tol = tol

	vector = np.zeros((shape[0],))
	v_j = 0
	vecmin = np.amin(B.arr[0])
	for i in range(shape[0]):
		if bNonZeros[i]:
			vector[i] = B.arr[0][v_j]
			v_j = v_j + 1
		else:
			vector[i] = vecmin

	del A
	del B
	del C
	del MA

	return vector

def LoopStaionaryDistributionRAM(matrix, A, B, C, mshape, c, nc):
	if c==1:
		A.arr[:] = MatrixPowerRAM(matrix, nc-1)[:]
	else:
		C.arr[:] = MatrixPowerRAM(matrix, nc-1)[:]
		A.arr[:] = mdot(B.arr, C.arr)
	B.arr[:] = mdot(A.arr, matrix)[:]

	#B.arr[:] = np.linalg.matrix_power(matrix, nc)[:]
	#A.arr[:] = np.dot(B.arr, matrix)[:]
	tidx = np.random.randint(mshape[0]-1, size=1)
	tol = np.amax((B.arr[tidx] - A.arr[tidx])**2)

	return tol

def LoopStaionaryDistributionHDD(MA, A, B, C, mshape, c, nc):
	if c==1:
		MatrixPowerHDD(MA, nc-1, result=A)
	else:
		MatrixPowerHDD(MA, nc-1, result=C)
		MatrixMultiplyHDD(B, C, Out=A)
	MatrixMultiplyHDD(A, MA, Out=B)

	tidx = np.random.randint(mshape[0]-1, size=1)
	tol = np.amax((B.arr[tidx] - A.arr[tidx])**2)

	return tol

def CalcStationaryDistributionByEigen(orig_matrix, shape, bNoData):
	matrix = (orig_matrix[~bNoData,:])[:,~bNoData]   # Selected row-column which are not all zeros
	mshape = matrix.shape

	A = MemoryMappedArray(mshape, dtype='float64')
	A.arr[:] = matrix[:]

	# Check if minimum is zero, replace all zeros with minimum value
	ma = np.ma.masked_equal(A.arr, 0.0, copy=False)
	idx = np.nonzero( A.arr == 0 )
	A.arr[idx] = ma.min()

	# Try using scipy sparse package because it is very fast
	#eigvalue, eigvector = sparse_linalg.eigs(A.arr.T, k=1, sigma=1.000)
	#eigvector = np.array(eigvector.flatten(), dtype=np.float64)
	#distrib = eigvector

	# Check if any egienvector value is negative, if yes, re-calculate eigenvector using scipy linear algebra package
	#idx = np.nonzero( distrib < 0 )[0]
	#if idx.any():
	eigvalue, eigvector = 	sp_linalg.eig(A.arr.T)
	distrib = eigvector[:,0]

	distrib = distrib/distrib.sum()

	prob_eigenvector = np.zeros((shape[0],))
	vecmin = np.amin(distrib)

	j = 0
	for i in range(shape[0]):
		if bNoData[i]:
			prob_eigenvector[i] = vecmin
		else:
			prob_eigenvector[i] = distrib[j]
			j = j + 1

	return prob_eigenvector/prob_eigenvector.sum()


def make_smooth_map(matrix, shape, filter_pass=1):

	def calc_mean(A, B, i, j, symmetric):
		istart = i-1
		iend = i+2
		jstart = j-1
		jend = j+2

		if istart < 0:
			istart = 1
		if jstart < 0:
			jstart = 1
		if iend > shape[0]:
			iend = shape[0]
		if jend > shape[1]:
			jend = shape[1]

		value = A.arr[istart:iend, jstart:jend].mean()
		B.arr[i][j] = value
		if symmetric:
			B.arr[j][i] = value


	A = MemoryMappedArray(shape, dtype='float32')
	A.arr[:] = matrix[:]
	B = MemoryMappedArray(shape, dtype='float32')

	symmetric = False
	if np.allclose(matrix.T, matrix):
		symmetric = True

	pass_num = 1
	while(pass_num <= filter_pass):
		for i in range(shape[0]):
			if symmetric:
				for j in range(i+1):
					calc_mean(A, B, i, j, symmetric)
			else:
				for j in range(shape[1]):
					calc_mean(A, B, i, j, symmetric)

			A.arr.flush()
			B.arr.flush()

		pass_num += 1
		A.arr[:] = B.arr[:]

	matrix[:] = A.arr[:]
	del A
	del B

def get_min_max_values(matrix, get_bNoData=False):
	bData = np.empty(shape=matrix.shape[0], dtype='bool')

	count = 0
	mx = matrix.shape[0]
	my = matrix.shape[1]

	for i in range(mx):
		if np.sum(matrix[i]) == 0.0:
			bData[i] = False
		else:
			count += 1
			bData[i] = True

	bData = np.asarray(bData)

	ci = -1
	cj = 0
	maxvalue = -9e10
	minvalue = 9e10
	for i in range(mx):
		if not bData[i]:
			continue
		for j in range(my):
			s = np.sort(matrix[i, np.nonzero(matrix[i])])[0]
			minvalue = min(minvalue, s[0])
			maxvalue = max(maxvalue, s[-1])

	if get_bNoData:
		return minvalue, maxvalue, ~bData
	else:
		return minvalue, maxvalue

def name_generator(size=10, chars=string.ascii_letters + string.digits ):
	s = ''.join(random.choice(chars) for _ in range(size))
	return s



#!/usr/bin/env python3
#
# Author: Rajendra Kumar
#
# This file is part of gcMapExplorer
# Copyright (C) 2016-2017  Rajendra Kumar, Ludvig Lizana, Per Stenberg
#
# gcMapExplorer is a free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gcMapExplorer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gcMapExplorer.  If not, see <http://www.gnu.org/licenses/>.
#
#=============================================================================

import os
import logging
import numpy as np
from scipy import stats
import numpy.ma as ma
import h5py

from . import ccmap as cmp
from . import gcmap as gmp
from . import ccmapHelpers as cmh
from . import genomicsDataHandler as gdh

from gcMapExplorer.config import getConfig

# get configuration
config = getConfig()

def correlateCMaps(ccMapObjOne, ccMapObjTwo, ignore_triangular=True, diagonal_offset=1, corrType='pearson', blockSize=None, slideStepSize=1, cutoffPercentile=None, workDir=None, outFile=None, logHandler=None):
	"""To calculate correlation between two Hi-C maps

	This function can be used to calculate either Pearson or Spearman rank-order correlation
	between two Hi-C maps. It also ignore lower-trangular matrix with diagnonal offset to avoid duplicate and large values.


	Parameters
	----------
	ccMapObjOne : :class:`gcMapExplorer.lib.ccmap.CCMAP`
		First :class:`gcMapExplorer.lib.ccmap.CCMAP` instance containing Hi-C data
	ccMapObjTwo : :class:`gcMapExplorer.lib.ccmap.CCMAP`
		Second :class:`gcMapExplorer.lib.ccmap.CCMAP` instance containing Hi-C data
	ignore_triangular : bool
		Whether entire matrix is considered or only one half triangular region of matrixis considered.
	diagonal_offset : int
		If ``ignore_triangular=True``, it is used to determine how much bins are ignored from the diagonal in one half triangular region of matrix.
		``diagonal_offset = 0`` is the main diagonal, ``diagonal_offset > 0`` means ignore this many bins from the diagonal.
	corrType : str
		Correlation type. For Pearson and Spearman rank-order correlation, use ``pearson`` and ``spearman``, respectively.
	blockSize : str
		To calculate block-wise correlations by sliding block of given size along diagonals. It should be in resolution.
		For example, ``1mb``, ``500kb``, ``5mb``, ``2.5mb`` etc.
		If ``None``, correlation of whole map is calculated. Sliding step of block depends on ``slideStepSize``.
	slideStepSize : int
	  	Step-size in bins by which blocks will be shifted for block-wise correlation. If slideStepSize is large then blocks might not be overlapped.
	workDir : str
		Name of working directory, where temporary files will be kept.If ``workDir = None``, file will be generated in OS based temporary directory.
	outFile : str
		Name of output file. Only written for block-wise correlation.


	Returns
	-------
	corr : float or list
		Correlation coefficient
	pvalue/centers : float or list
		If ``blockSize=None`` 2-tailed p-value is returned. For block-wise correlation, list of block-center is returned.


	.. seealso::

		* `scipy.stats.stats.pearsonr <http://docs.scipy.org/doc/scipy/reference/generated/scipy.stats.pearsonr.html#scipy.stats.pearsonr>`_ for Pearson correlation.
		* `scipy.stats.stats.spearmanr <http://docs.scipy.org/doc/scipy/reference/generated/scipy.stats.spearmanr.html#scipy.stats.spearmanr>`_ for Spearman rank-order correlation.



	"""
	# logger
	logger = logging.getLogger('correlateCMaps')
	if logHandler is not None:
		logger.propagate = False
		logger.addHandler(logHandler)
	logger.setLevel(logging.INFO)

	# To check, user have given correct keywords
	if not (corrType == 'pearson' or corrType == 'spearman' or corrType == 'kendell-tau'):
		raise NotImplementedError ('{0} not implemented. Use "pearson" or "spearman".' .format(corrType))

	# To check if resolution of maps are same
	if ccMapObjOne.binsize != ccMapObjTwo.binsize:
		raise AssertionError ("Resolution of first Hi-C map does not match with Second Hi-C map.")

	# Determine smallest shape between two maps
	if ccMapObjOne.shape[0] >= ccMapObjTwo.shape[0]:
		smallest_shape = ccMapObjTwo.shape[0]
	else:
		smallest_shape = ccMapObjOne.shape[0]

	# Calculation for whole map
	if blockSize is not None:
		logger.info(' Block-wise correlation with [{0}] block-size'.format(blockSize))

		# checking if block-size is larger than smallest shape
		size = int(cmp.resolutionToBinsize(blockSize)/ccMapObjOne.binsize)
		if size >= smallest_shape:
			raise AssertionError ("Size of input block [{0}] is larger than smallest size [{1}] of Hi-C map".format(size, smallest_shape))
		else:
			blockSize = size

		# Print some information about blocks
		logger.info(' Number of Blocks: {0} '.format( int(smallest_shape/blockSize) ) )
		logger.info(' Size of each Block in bins: {0} '.format( blockSize ) )
		if size-slideStepSize < 0:
			logger.info(' Blocks are not overlapping. Distance between two adjacent blocks in bins : {0}'.format(slideStepSize-size))
		else:
			logger.info(' Number of Overlapping bins between sliding blocks:  {0}' .format( size-slideStepSize ) )


	corr, pvalue, centers = None, None, None

	try:

		# generate boolean arrays to store mask
		m1 = cmh.MemoryMappedArray(shape=(smallest_shape, smallest_shape), dtype=np.bool, workDir=workDir)
		m2 = cmh.MemoryMappedArray(shape=(smallest_shape, smallest_shape), dtype=np.bool, workDir=workDir)
		mask = cmh.MemoryMappedArray(shape=(smallest_shape, smallest_shape), dtype=np.bool, workDir=workDir)

		# Determine masks for two maps and combine it
		m1.arr[:] = ccMapObjOne.matrix[:smallest_shape, :smallest_shape] <= ccMapObjOne.minvalue
		m2.arr[:] = ccMapObjTwo.matrix[:smallest_shape, :smallest_shape] <= ccMapObjTwo.minvalue

		if cutoffPercentile is not None:
			percentileOne = np.percentile(ma.array(ccMapObjOne.matrix[:smallest_shape, :smallest_shape], mask=m1.arr).compressed(), cutoffPercentile)
			percentileTwo = np.percentile(ma.array(ccMapObjTwo.matrix[:smallest_shape, :smallest_shape], mask=m2.arr).compressed(), cutoffPercentile)
			m1.arr[:] = ccMapObjOne.matrix[:smallest_shape, :smallest_shape] > percentileOne
			m2.arr[:] = ccMapObjTwo.matrix[:smallest_shape, :smallest_shape] > percentileTwo

			mask.arr[:] = ~( m1.arr | m2.arr )
			mask.arr[ np.nonzero( ccMapObjOne.matrix[:smallest_shape, :smallest_shape] == 0.0) ] = True
			mask.arr[ np.nonzero( ccMapObjTwo.matrix[:smallest_shape, :smallest_shape] == 0.0) ] = True
		else:
			mask.arr[:] = ( m1.arr | m2.arr )

		# Mask lower diagonal with diagonal_offset
		if ignore_triangular:
			mask.arr[np.tril_indices_from(mask.arr, k=diagonal_offset)] = True

		if blockSize is None:
			# This section for correlation between whole maps
			ccMapObjOneMa = ma.array(ccMapObjOne.matrix[:smallest_shape, :smallest_shape], mask=mask.arr)
			ccMapObjTwoMa = ma.array(ccMapObjTwo.matrix[:smallest_shape, :smallest_shape], mask=mask.arr)

			if ccMapObjOneMa[~ccMapObjOneMa.mask].shape[0] > 10:
				if corrType == 'pearson':
					corr, pvalue = stats.pearsonr(ccMapObjOneMa.compressed(), ccMapObjTwoMa.compressed())
				elif corrType == 'kendall-tau':
					corr, pvalue = stats.kendelltau(ccMapObjOneMa.compressed(), ccMapObjTwoMa.compressed())
				else:
					corr, pvalue = stats.spearmanr(ccMapObjOneMa.compressed(), ccMapObjTwoMa.compressed())

			else:
				corr = 0
				pvalue = 0

		else:
			# This section for block-wise correlation between maps
			corr, centers = [], []
			csp = 0           # Current start position
			cep = blockSize   # Current end position
			while(cep < smallest_shape):

				ccMapObjOneMa = ma.array(ccMapObjOne.matrix[csp:cep, csp:cep], mask=mask.arr[csp:cep, csp:cep])
				ccMapObjTwoMa = ma.array(ccMapObjTwo.matrix[csp:cep, csp:cep], mask=mask.arr[csp:cep, csp:cep])

				# At least 10 valid values are present
				if ccMapObjOneMa[~ccMapObjOneMa.mask].shape[0] > 10:
					if corrType == 'pearson':
						tcorr, tpvalue = stats.pearsonr(ccMapObjOneMa.compressed(), ccMapObjTwoMa.compressed())
					elif corrType == 'kendall-tau':
						corr, pvalue = stats.kendelltau(ccMapObjOneMa.compressed(), ccMapObjTwoMa.compressed())
					else:
						tcorr, tpvalue = stats.spearmanr(ccMapObjOneMa.compressed(), ccMapObjTwoMa.compressed())
				else:
					tcorr = 0

				if tcorr is ma.masked:
					corr.append(0.0)
				else:
					corr.append(float(tcorr))

				centers.append( (csp + (blockSize/2) ) * ccMapObjOne.binsize )

				csp = csp + slideStepSize
				cep = csp + blockSize

		del m1
		del m2
		del mask
	except (KeyboardInterrupt, SystemExit) as e:
		del m1
		del m2
		del mask
		raise e

	if logHandler is not None:
		logger.removeHandler( logHandler )

	if outFile is not None and blockSize is not None:
		fout = open(outFile, 'w')
		for i in range(len(corr)):
			if corr[i] != 0:
				fout.write('{0}\t{1}\n'.format(int(centers[i]), corr[i]))
		fout.close()


	if blockSize is None:
		return corr, pvalue
	else:
		return corr, centers


def correlateGCMaps(gcmapOne, gcmapTwo, outFile=None, blockSize=None, slideStepSize=1, name=None, cutoffPercentile=None, ignore_triangular=True, diagonal_offset=1, corrType='pearson', workDir=None, logHandler=None):

	# logger
	logger = logging.getLogger('correlateGCMaps')
	if logHandler is not None:
		logger.propagate = False
		logger.addHandler(logHandler)
	logger.setLevel(logging.INFO)

	if blockSize is not None:
		if outFile is None:
			raise ValueError('No output file provided.')
		if name is None:
			raise ValueError('No data name provided.')

	mapListOne = []
	hdf5 = h5py.File(gcmapTwo)
	for key in hdf5:
		if hdf5[key].attrs['xlabel'] == hdf5[key].attrs['ylabel']:
			mapListOne.append(key)
	hdf5.close()

	mapListTwo = []
	hdf5 = h5py.File(gcmapOne)
	for key in hdf5:
		if hdf5[key].attrs['xlabel'] == hdf5[key].attrs['ylabel']:
			mapListTwo.append(key)
	hdf5.close()

	mapList = cmh.sorted_nicely( list(set(mapListOne).intersection(mapListTwo)) )

	if blockSize is not None:
		h5Handle = gdh.HDF5Handler(outFile)
	else:
		corrs, pvalues = [], []

	for key in mapList:

		logger.info(' Performing calculation for {0} ...'.format(key))


		ccMapObjOne = gmp.loadGCMapAsCCMap(gcmapOne, mapName=key, workDir=workDir)
		ccMapObjOne.make_readable()

		ccMapObjTwo = gmp.loadGCMapAsCCMap(gcmapTwo, mapName=key, workDir=workDir)
		ccMapObjTwo.make_readable()


		#ccMapObjOne = gmp.GCMAP(gcmapOne, mapName=key)
		#ccMapObjTwo = gmp.GCMAP(gcmapTwo, mapName=key)

		binsize = ccMapObjOne.binsize

		dataArray = np.zeros(shape=(max(ccMapObjOne.shape[0], ccMapObjTwo.shape[0]),) )

		try:
			results = correlateCMaps(ccMapObjOne, ccMapObjTwo, ignore_triangular=ignore_triangular, diagonal_offset=diagonal_offset, corrType=corrType,
		                blockSize=blockSize, cutoffPercentile=cutoffPercentile, slideStepSize=slideStepSize, workDir=workDir, outFile=None, logHandler=logHandler)
		except	(KeyboardInterrupt, SystemExit) as e:
			del ccMapObjOne
			del ccMapObjTwo
			raise e

		del ccMapObjOne
		del ccMapObjTwo

		corr, centers = None, None
		if results is not None:
			corr, centers = results

		if blockSize is not None:
			centers = np.asarray(centers) / binsize
			centers = centers.astype(int)
			dataArray[(centers,)] = corr
			h5Handle._addDataByArray(key, cmp.binsizeToResolution(binsize), name, dataArray)
		else:
			corrs.append(corr)
			pvalues.append(centers)

		logger.info('     Finished calculation for {0} ...\n'.format(key))

	if blockSize is not None:
		h5Handle.close()

	if blockSize is None:
		return mapList, corrs, pvalues


def correlateCMapsBinWise(ccMapObjOne, ccMapObjTwo, corrType='pearson', cutoffPercentile=None, workDir=None, outFile=None, logHandler=None):
	"""To calculate correlation between two Hi-C maps

	This function can be used to calculate either Pearson or Spearman rank-order correlation
	between two Hi-C maps. It also ignore lower-trangular matrix with diagnonal offset to avoid duplicate and large values.


	Parameters
	----------
	ccMapObjOne : :class:`gcMapExplorer.lib.ccmap.CCMAP`
		First :class:`gcMapExplorer.lib.ccmap.CCMAP` instance containing Hi-C data
	ccMapObjTwo : :class:`gcMapExplorer.lib.ccmap.CCMAP`
		Second :class:`gcMapExplorer.lib.ccmap.CCMAP` instance containing Hi-C data
	ignore_triangular : bool
		Whether entire matrix is considered or only one half triangular region of matrixis considered.
	diagonal_offset : int
		If ``ignore_triangular=True``, it is used to determine how much bins are ignored from the diagonal in one half triangular region of matrix.

		``diagonal_offset = 0`` is the main diagonal, ``diagonal_offset > 0`` means ignore this many bins from the diagonal.

	corrType : str
		Correlation type. For Pearson and Spearman rank-order correlation, use ``pearson`` and ``spearman``, respectively.
	blockSize : str
		To calculate block-wise correlations by sliding block of given size along diagonals. It should be in resolution.
		For example, ``1mb``, ``500kb``, ``5mb``, ``2.5mb`` etc.
		If ``None``, correlation of whole map is calculated. Sliding step of block depends on ``slideStepSize``.
	slideStepSize : int
	  	Step-size in bins by which blocks will be shifted for block-wise correlation. If slideStepSize is large then blocks might not be overlapped.

	workDir : str
		Name of working directory, where temporary files will be kept.If ``workDir = None``, file will be generated in OS based temporary directory.

	outFile : str
		Name of output file. Only written for block-wise correlation.

	Returns
	-------
	corr : float or list
		Correlation coefficient
	pvalue/centers : float or list
		If ``blockSize=None`` 2-tailed p-value is returned. For block-wise correlation, list of block-center is returned.


	.. seealso::
		* `scipy.stats.stats.pearsonr <http://docs.scipy.org/doc/scipy/reference/generated/scipy.stats.pearsonr.html#scipy.stats.pearsonr>`_ for Pearson correlation.
		* `scipy.stats.stats.spearmanr <http://docs.scipy.org/doc/scipy/reference/generated/scipy.stats.spearmanr.html#scipy.stats.spearmanr>`_ for Spearman rank-order correlation.


	"""
	# logger
	logger = logging.getLogger('correlateCMaps')
	if logHandler is not None:
		logger.propagate = False
		logger.addHandler(logHandler)
	logger.setLevel(logging.INFO)

	# To check, user have given correct keywords
	if not (corrType == 'pearson' or corrType == 'spearman' or corrType == 'kendell-tau'):
		raise NotImplementedError ('{0} not implemented. Use "pearson" or "spearman".' .format(corrType))

	# To check if resolution of maps are same
	if ccMapObjOne.binsize != ccMapObjTwo.binsize:
		raise AssertionError ("Resolution of first Hi-C map does not match with Second Hi-C map.")

	# Determine smallest shape between two maps
	if ccMapObjOne.shape[0] >= ccMapObjTwo.shape[0]:
		smallest_shape = ccMapObjTwo.shape[0]
	else:
		smallest_shape = ccMapObjOne.shape[0]

	corr, centers = [], []
	try:

		# generate boolean arrays to store mask
		m1 = cmh.MemoryMappedArray(shape=(smallest_shape, smallest_shape), dtype=np.bool, workDir=workDir)
		m2 = cmh.MemoryMappedArray(shape=(smallest_shape, smallest_shape), dtype=np.bool, workDir=workDir)
		mask = cmh.MemoryMappedArray(shape=(smallest_shape, smallest_shape), dtype=np.bool, workDir=workDir)

		# Determine masks for two maps and combine it
		m1.arr[:] = ccMapObjOne.matrix[:smallest_shape, :smallest_shape] == ccMapObjOne.minvalue
		m2.arr[:] = ccMapObjTwo.matrix[:smallest_shape, :smallest_shape] == ccMapObjTwo.minvalue

		if cutoffPercentile is not None:
			percentileOne = np.percentile(ma.array(ccMapObjOne.matrix[:smallest_shape, :smallest_shape], mask=m1.arr).compressed(), cutoffPercentile)
			percentileTwo = np.percentile(ma.array(ccMapObjTwo.matrix[:smallest_shape, :smallest_shape], mask=m2.arr).compressed(), cutoffPercentile)
			m1.arr[:] = ccMapObjOne.matrix[:smallest_shape, :smallest_shape] <= percentileOne
			m2.arr[:] = ccMapObjTwo.matrix[:smallest_shape, :smallest_shape] <= percentileTwo

		mask.arr[:] = ( m1.arr | m2.arr )

		for b in range(smallest_shape):
			ccMapObjOneMa = ma.array(ccMapObjOne.matrix[b], mask=mask.arr[b])
			ccMapObjTwoMa = ma.array(ccMapObjTwo.matrix[b], mask=mask.arr[b])

			tcorr = 0
			if ccMapObjOneMa[~ccMapObjOneMa.mask].shape[0] > 10:
				if corrType == 'pearson':
					tcorr, _ = stats.pearsonr(ccMapObjOneMa.compressed(), ccMapObjTwoMa.compressed())
				elif corrType == 'kendall-tau':
					tcorr, _ = stats.kendelltau(ccMapObjOneMa.compressed(), ccMapObjTwoMa.compressed())
				else:
					tcorr, _ = stats.spearmanr(ccMapObjOneMa.compressed(), ccMapObjTwoMa.compressed())

			if tcorr is ma.masked:
				corr.append(0.0)
			else:
				corr.append(float(tcorr))
			centers.append( b * ccMapObjOne.binsize )

		del m1
		del m2
		del mask
	except (KeyboardInterrupt, SystemExit) as e:
		del m1
		del m2
		del mask
		raise e

	if logHandler is not None:
		logger.removeHandler( logHandler )

	if outFile is not None:
		fout = open(outFile, 'w')
		for i in range(len(corr)):
			if corr[i] != 0:
				fout.write('{0}\t{1}\n'.format(int(centers[i]), corr[i]))
		fout.close()

	return corr, centers

def _addDataToH5(bins, values, binsize, shape, fileName, chromosome, dataName):
	h5Handle = gdh.HDF5Handler(fileName)
	dataArray = np.zeros(shape=(shape,))
	idx = np.asarray(bins) / binsize
	idx = idx.astype(int)
	dataArray[(idx,)] = values
	h5Handle._addDataByArray(chromosome, cmp.binsizeToResolution(binsize), dataName, dataArray)
	h5Handle.close()


def getAvgContactByDistance(ccmaps, stats='median'):
    """To calcualte average contact as a function of distance

    Parameters
    ----------
    hicmaps : :class:`gcMapExplorer.lib.ccmap.CCMAP` or list[:class:`gcMapExplorer.lib.ccmap.CCMAP`]

    Returns
    -------
    avg_contacts : numpy.array
    	A one-dimensional numpy array containing average contacts, where index is distance between two locations for given resolution/binsize.
    	For example, if ``ccmap.binsize=100000`` and ``avg_contacts[4]=1234.56``, then at distance of 400000 b, average contact is ``1234.56``.

    """
    ccmapList = []
    if isinstance(ccmaps, cmp.CCMAP) or isinstance(ccmaps, gmp.GCMAP):
        ccmapList = [ccmaps]
    elif isinstance(ccmaps, list):
        for ccmap in ccmaps:
            if not isinstance(ccmap, cmp.CCMAP):
                raise TypeError ('Not a list of CCMAP instances')

        ccmapList = ccmaps

    else:
        raise TypeError ('Not a CCMAP instance or a list of CCMAP instances')

    # Determine max shape
    largest_shape = 0
    for ccmap in ccmapList:
        largest_shape = max(ccmap.shape[0], largest_shape)

    # Start calculation stats
    avg_contacts = []
    for i in range(largest_shape):

        data = np.empty(0)

        for t in range(len(ccmapList)):
            idx = cmh.kth_diag_indices(i, ccmapList[t].matrix)

            # Mask all zero values, then calculate stats
            marray = ma.masked_equal(ccmapList[t].matrix[idx], 0.0, copy=False)
            data = np.hstack((marray.compressed(), data))

        if data.shape[0] >= 1:
            if stats == 'median':
                t_avg = np.median(data)
            else:
                t_avg = np.mean(data)

            avg_contacts.append( t_avg )

        else:
            avg_contacts.append( 0.0 )

        del data

    return np.asarray(avg_contacts)



#!/usr/bin/env python3
#
# Author: Rajendra Kumar
#
# This file is part of gcMapExplorer
# Copyright (C) 2016-2017  Rajendra Kumar, Ludvig Lizana, Per Stenberg
#
# gcMapExplorer is a free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gcMapExplorer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gcMapExplorer.  If not, see <http://www.gnu.org/licenses/>.
#
#=============================================================================

import os, re
import shlex
import sys
import string, random

from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5.QtPrintSupport import *
from PyQt5.uic import loadUiType

from gcMapExplorer import ccmap as cmp
from gcMapExplorer import gcmap as gmp
from gcMapExplorer import ccmapHelpers as cmh
from gcMapExplorer import genomicsDataHandler as gdh

from gcMapExplorer.config import getConfig

# get configuration
config = getConfig()

# Determine absolute path to UIs directory. Relative path from this directory does not work.
DirToThisScript = os.path.dirname(os.path.abspath(__file__))
PathToUIs = os.path.join(DirToThisScript, 'UIs')

def main():
    app = QApplication(sys.argv)
    converter_interface = DialogOther1DFormatLoader()
    converter_interface.setAttribute(Qt.WA_DeleteOnClose)
    converter_interface.show()
    converter_interface.removeH5RemoveRadioButton()
    app.exec_()
    app.exit()

# Dialog box to change page size by custom length
pathToThisUI = os.path.join(PathToUIs, 'other1DFormatFileConversion.ui')
Ui_DialogOther1DFormatLoader, DialogOther1DFormatLoaderBase = loadUiType(pathToThisUI)
class DialogOther1DFormatLoader(DialogOther1DFormatLoaderBase, Ui_DialogOther1DFormatLoader):
    def __init__(self, parent=None):
        super(DialogOther1DFormatLoader, self).__init__(parent=parent)
        self.setupUi(self)

        # Resize hight and reduce size of log text box
        self.resize(self.width(), 600)
        self.splitter.setSizes([460, 140])
        self.command = None
        self.process = None
        self.hideMode = False
        self.indexFile = None
        self.hasConvertedSuccessfully = False


        # Hide tabbars
        tabbars = self.fileFormatSpecificTabWidget.findChildren(QTabBar)
        for tabbar in tabbars:
            tabbar.hide()

        # Set default scratch directory, bigWigInfo and bigWigToWig paths
        self.setDefaultScratchDirs()
        self.setBigWigInfoPathConfig()
        self.setBigWigToWigPathConfig()


        self.inFileButton.setIcon( self.style().standardIcon(QStyle.SP_DirOpenIcon) )
        self.bigWigToWigPathButton.setIcon( self.style().standardIcon(QStyle.SP_DirOpenIcon) )
        self.bigWigInfoButton.setIcon( self.style().standardIcon(QStyle.SP_DirOpenIcon) )
        self.h5FileButton.setIcon( self.style().standardIcon(QStyle.SP_DirOpenIcon) )
        self.scratchDirButton.setIcon( self.style().standardIcon(QStyle.SP_DirOpenIcon) )

        self.ameanCheckBox.clicked.connect( self.checkDownsamplingMethods )
        self.hmeanCheckBox.clicked.connect( self.checkDownsamplingMethods )
        self.gmeanCheckBox.clicked.connect( self.checkDownsamplingMethods )
        self.minCheckBox.clicked.connect( self.checkDownsamplingMethods )
        self.maxCheckBox.clicked.connect( self.checkDownsamplingMethods )
        self.medianCheckBox.clicked.connect( self.checkDownsamplingMethods )

        self.inFileButton.clicked.connect( self.browseInputFile )
        self.fileFormatCBox.currentIndexChanged.connect( self.fileFormatSpecificTabWidget.setCurrentIndex )
        self.scratchDirButton.clicked.connect( self.browseScratchDir )
        self.h5FileButton.clicked.connect( self.browseH5File )
        if hasattr(self, 'h5RemoveRadioButton'):
            self.h5RemoveRadioButton.clicked.connect( self.isValidH5OutFileFromWidget )
        self.bigWigInfoButton.clicked.connect( self.browseBigWigInfoProgram )
        self.bigWigToWigPathButton.clicked.connect( self.browseBigWigToWigProgram )
        self.logClearButton.clicked.connect( self.logOutputPlainTextEdit.clear )

        self.inFileLineEdit.editingFinished.connect( self.checkInputFileFormat )
        self.h5FileLineEdit.editingFinished.connect( self.isValidH5OutFileFromWidget )
        self.scratchDirLineEdit.editingFinished.connect( self.checkScratchDirectory )
        self.resolutionLineEdit.editingFinished.connect( self.checkResolutions )
        self.bigWigInfoLineEdit.editingFinished.connect( self.checkBigWigInfoProgram )
        self.bigWigToWigPathLineEdit.editingFinished.connect( self.checkBigWigToWigProgram )

        self.whatIsThisButton.clicked.connect( QWhatsThis.enterWhatsThisMode )
        self.convertButton.clicked.connect( self.startProcess )
        self.stopButton.clicked.connect( self.terminateProcessing )
        self.closeButton.clicked.connect( self.close )

    def closeEvent(self, event):

        # if process is running
        if self.process is not None:
            msg = 'Conversion is still running...\n First stop the process.'
            showWarningMessageBox(msg, self)
            event.ignore()
            return

        # In case when hide mode is on, do not close, but hide
        if self.hideMode:
            event.ignore()
            self.hide()
            return

        # Remove temporary h5 file
        if hasattr(self, 'h5RemoveRadioButton'):
            if self.h5RemoveRadioButton.isChecked() and \
                                            self.h5FileLineEdit.text():
                try:
                    os.remove( self.h5FileLineEdit.text() )
                except:
                    pass

        # Remove index file
        if self.indexFile is not None:
            try:
                os.remove(self.indexFile)
            except:
                pass

    def connet2Hide(self):
        """ When opened from browser, hide dialog on close
        """
        try:
            self.closeButton.clicked.disconnect()
        except:
            pass
        self.closeButton.clicked.connect( self.hideDialog )
        self.hideMode = True

    def hideDialog(self):
        if self.process is not None:
            msg = 'Conversion is still running...\n First stop the process.'
            showWarningMessageBox(msg, self)
        else:
            self.hide()

    def removeH5RemoveRadioButton(self):
        """ Remove h5RemoveRadioButton from form.
        It can be useful if this GUI is executed standalone instead from the
        browser
        """
        self.h5RemoveRadioButton.setParent(None)
        del self.h5RemoveRadioButton
        self.h5Line.setParent( None )
        del self.h5Line

    def setH5Name(self, name=None):
        """Set h5 file name.
        In case if h5 name is not provided a random name is generated.
        Also, return the name.
        """
        if name is None:
            self.genRandomH5Name()
        else:
            self.h5FileLineEdit.setText( name )

        return self.h5FileLineEdit.text()

    def setChromName(self, chromName=None):
        """ Set the input chromosome name.
        """
        if chromName is not None:
            self.chromNameLineEdit.setText(chromName)
            self.hasConvertedSuccessfully = False
            self.command = None

    def setInputFile(self, inputFile):
        """Set the input file through code
        """
        self.inFileLineEdit.setText(inputFile)
        self.checkInputFileFormat()

    def enableFileIndexing(self):
        """Generate index file
        """
        chars = string.ascii_letters + string.digits
        name = 	''.join(random.choice(chars) for _ in range(8))
        name = name + '.json'
        self.indexFile = os.path.join( self.scratchDirLineEdit.text(), name )

    def setDefaultScratchDirs(self):
        """ Set default scratch directory
        """
        defaultDir = config['Dirs']['WorkingDirectory']
        self.scratchDirLineEdit.setText(defaultDir)

    def setBigWigInfoPathConfig(self):
        """ Set path of bigWigInfo program from configuration
        """
        path = config['Programs']['bigWigInfo']
        if path != 'None':
            self.bigWigInfoLineEdit.setText( path )

    def setBigWigToWigPathConfig(self):
        """ Set path of bigWigToWig program from configuration
        """
        path = config['Programs']['bigWigToWig']
        if path != 'None':
            self.bigWigToWigPathLineEdit.setText( path )

    def browseScratchDir(self):
        """Browse and choose scratch directory
        """
        path = QFileDialog.getExistingDirectory(self, 'Select Scratch Directory')
        if path:
            self.scratchDirLineEdit.setText(path)
        self.checkScratchDirectory(fromButton=True)

    def checkScratchDirectory(self, fromButton=False):
        """ Check whether scratch directory is provided correctly in the
        line edit.
        Return: None or scracth directory name
        """
        if (self.scrollArea.focusWidget() is  not self.scratchDirButton) or (fromButton):
            checkDirExist(self.scratchDirLineEdit, self)
        scratchDir = self.scratchDirLineEdit.text()
        if scratchDir:
            return scratchDir
        else:
            if fromButton:
                msg = 'No scratch directory'
                showWarningMessageBox(msg, self)
                self.scratchDirLineEdit.selectAll()
                self.scratchDirLineEdit.setFocus()

    def genRandomH5Name(self):
        """ Generate random name of h5 file in scratch directory
        """
        chars = string.ascii_letters + string.digits
        name = 	''.join(random.choice(chars) for _ in range(8))
        name = name + '.h5'
        h5 = os.path.join( self.scratchDirLineEdit.text(), name )
        self.h5FileLineEdit.setText( h5 )

    def browseInputFile(self):
        """To get input file with full path
        """
        # A dialog box will be displayed to select a file and path will be stored in the cell
        file_choices = " dataset files (*.bigWig *.wig *.bed);;All Files(*.*)"
        path = QFileDialog.getOpenFileName(self, 'Open File', '', file_choices)
        if path[0]:
            self.inFileLineEdit.setText(path[0])
        self.checkInputFileFormat(fromButton=True)

    def checkInputFileFormat(self, fromButton=False):
        """ Check file extension and accessibility of input file.
        """
        if (self.scrollArea.focusWidget() is  not self.inFileButton) or (fromButton):
            fileExist = checkFileExist(self.inFileLineEdit, self)
            if not fileExist:
                return

        fileName = self.inFileLineEdit.text()
        name = os.path.basename( fileName )
        if name and ((self.scrollArea.focusWidget() is  not self.inFileButton) \
                        or (fromButton)):
            extension = os.path.splitext( name )[1]
            fileOK = True
            if extension.lower() == '.bigwig':
                self.fileFormatCBox.setCurrentIndex( 0 )
            elif extension.lower() == '.wig':
                self.fileFormatCBox.setCurrentIndex( 1 )
            elif extension.lower() == '.bed':
                self.fileFormatCBox.setCurrentIndex( 2 )
            else:
                msg = 'File extension [{0}] is not acceptable.'.format(extension)
                showWarningMessageBox(msg, self)
                self.inFileLineEdit.selectAll()
                self.inFileLineEdit.setFocus()
                fileOK = False

            if fileOK:
                return fileName
            else:
                return

        else:
            return

    def browseH5File(self):
        """To get output file with full path
        """
        # A dialog box will be displayed to select a file and path will be stored in the cell
        file_choices = " h5/hdf5 files (*.h5 *.hdf5);;All Files(*.*)"
        path = QFileDialog.getSaveFileName(self, 'Select or Create File', '', file_choices, options=QFileDialog.DontConfirmOverwrite)
        if path[0]:
            self.h5FileLineEdit.setText(path[0])
        self.validateH5OutFile()

    def validateH5OutFile(self):
        """ This can be used to validate internally.
        Return True or None
        """
        h5Out = self.h5FileLineEdit.text()
        if not h5Out:
            return

        h5OutDir = os.path.dirname( self.h5FileLineEdit.text() )
        h5BaseName = os.path.basename( self.h5FileLineEdit.text() )
        h5Extension = os.path.splitext( h5BaseName )[1]
        scratchDir = self.scratchDirLineEdit.text()

        if hasattr(self, 'h5RemoveRadioButton'):
            if (scratchDir == h5OutDir) and not self.h5RemoveRadioButton.isChecked():
                msg = 'You are trying to save h5 file in scratch directory.\n\
                       Please, use another directory.'
                showWarningMessageBox(msg, self)
                self.h5FileLineEdit.selectAll()
                self.h5FileLineEdit.setFocus()
                return
        else:
            if (scratchDir == h5OutDir):
                msg = 'You are trying to save h5 file in scratch directory.\n\
                       Please, use another directory.'
                showWarningMessageBox(msg, self)
                self.h5FileLineEdit.selectAll()
                self.h5FileLineEdit.setFocus()
                return

        # In extension is different, change it to correct one
        if not (h5Extension == '.h5' or h5Extension == '.hdf5'):
            h5Out = os.path.join(h5OutDir, h5BaseName+'.h5')
            self.h5FileLineEdit.setText(h5Out)

        return self.h5FileLineEdit.text()

    def isValidH5OutFileFromWidget(self, checked=None):
        """ Check whether scratch directory and h5 directory is same.
        Also, check that path given in h5 box is valid.

        Return:
            If everything is fine: True otherwise: None
        """

        h5Out = self.h5FileLineEdit.text()
        if not h5Out:
            return

        h5OutDir = os.path.dirname( self.h5FileLineEdit.text() )
        h5BaseName = os.path.basename( self.h5FileLineEdit.text() )
        h5Extension = os.path.splitext( h5BaseName )[1]
        scratchDir = self.scratchDirLineEdit.text()

        # Check if h5outdir is available
        if not os.path.isdir( h5OutDir ):
            msg = 'Directory {0} not found.'.format(h5OutDir)
            showWarningMessageBox(msg, self)
            self.h5FileLineEdit.selectAll()
            self.h5FileLineEdit.setFocus()
            return


        # In case if there is no remove buttone make checked False.
        # Do not save file in scratch directory
        if not hasattr(self, 'h5RemoveRadioButton'):
            checked = False

        if (scratchDir == h5OutDir) and not checked and checked is not None\
            and (self.scrollArea.focusWidget() is  not self.h5FileButton):
            msg = 'You are trying to save h5 file in scratch directory.\n\
                   Please, use another directory.'
            showWarningMessageBox(msg, self)
            self.h5FileLineEdit.selectAll()
            self.h5FileLineEdit.setFocus()
            return

        # If Revome button is checked, then check if scratch directory
        # is not same to output directory
        # This is done only in case remove button is present
        if hasattr(self, 'h5RemoveRadioButton'):
            if self.h5RemoveRadioButton.isChecked():
                checked = True
            else:
                checked = False

            ## show only when user changes focus from line-edit to any
            ## other widget except remove button.
            ## User should be able to click on remove button
            if (scratchDir == h5OutDir) and (not checked) and \
            (self.scrollArea.focusWidget() is  not self.h5RemoveRadioButton) \
            and (self.scrollArea.focusWidget() is  not self.h5FileButton):
                msg = 'You are trying to save h5 file in scratch directory.\n\
                       Please, use another directory.'
                showWarningMessageBox(msg, self)
                self.h5FileLineEdit.selectAll()
                self.h5FileLineEdit.setFocus()
                return

        # In extension is different, change it to correct one
        if not (h5Extension == '.h5' or h5Extension == '.hdf5'):
            h5Out = os.path.join(h5OutDir, h5BaseName+'.h5')
            self.h5FileLineEdit.setText(h5Out)

        return True

    def checkResolutions(self):
        """ Check resolutions inputs.
        Rerturn resolutions list or None
        """
        rInList = self.resolutionLineEdit.text()
        rlist = []
        if rInList:
            temp = re.split(',', rInList)
            for r in temp:
                r = r.rstrip().lstrip()
                if not r.strip():
                    continue
                try:
                    b = cmp.resolutionToBinsize(r)
                    rlist.append( cmp.binsizeToResolution(b) )
                except ValueError:
                    msg = '"{0}" contains resolution "{1}"...\n which is not an acceptable resolution.'.format(rInList, r)
                    showWarningMessageBox(msg, self )

            return rlist

    def checkDownsamplingMethods(self, checked=None):
        """Check downsampling method.
        Return method list or None
        """
        methods = []
        if self.ameanCheckBox.isChecked():
            methods.append('amean')
        if self.hmeanCheckBox.isChecked():
            methods.append('hmean')
        if self.gmeanCheckBox.isChecked():
            methods.append('gmean')
        if self.maxCheckBox.isChecked():
            methods.append('max')
        if self.minCheckBox.isChecked():
            methods.append('min')
        if self.medianCheckBox.isChecked():
            methods.append('median')

        if not methods:
            msg = 'Please select at least one downsampling method.'
            showWarningMessageBox(msg, self )
            self.coarseGroupBox.setFocus()
            return None
        else:
            return methods

    def browseBigWigInfoProgram(self):
        """To get bigWigInfo path
        """
        # A dialog box will be displayed to select a file and path will be stored in the cell
        file_choices = " Binary Executable (* *.exe)"
        path = QFileDialog.getOpenFileName(self, 'Open File', '', file_choices)
        if path[0]:
            self.bigWigInfoLineEdit.setText(path[0])
        self.checkBigWigInfoProgram(fromButton=True)

    def checkBigWigInfoProgram(self, fromButton=False):
        """Check if path to bigWigInfo is valid
        """
        if (self.scrollArea.focusWidget() is  not self.bigWigInfoButton) or (fromButton):
            checkFileExist(self.bigWigInfoLineEdit, self)
        bigWigInfoProgram = self.bigWigInfoLineEdit.text()
        if bigWigInfoProgram:
            return bigWigInfoProgram
        else:
            if fromButton:
                msg = 'No bigWigInfo !!!'
                showWarningMessageBox(msg, self)
                self.bigWigInfoLineEdit.selectAll()
                self.bigWigInfoLineEdit.setFocus()

    def browseBigWigToWigProgram(self):
        """To get bigWigToWig path
        """
        # A dialog box will be displayed to select a file and path will be stored in the cell
        file_choices = " Binary Executable (*)"
        path = QFileDialog.getOpenFileName(self, 'Open File', '', file_choices)
        if path[0]:
            self.bigWigToWigPathLineEdit.setText(path[0])
        self.checkBigWigToWigProgram(fromButton=True)

    def checkBigWigToWigProgram(self, fromButton=False):
        """ Check path to bigWigToWig is valid
        """
        if (self.scrollArea.focusWidget() is  not self.bigWigToWigPathButton) or (fromButton):
            checkFileExist(self.bigWigToWigPathLineEdit, self)
        bigWigToWigProgram = self.bigWigToWigPathLineEdit.text()
        if bigWigToWigProgram:
            return bigWigToWigProgram
        else:
            if fromButton:
                msg = 'No bigWigToWig !!!'
                showWarningMessageBox(msg, self)
                self.bigWigToWigPathLineEdit.selectAll()
                self.bigWigToWigPathLineEdit.setFocus()

    def buildCommand(self):
        """ Build the command
        """

        self.command = None
        command = dict()

        # Input file
        command['-i'] = self.checkInputFileFormat(fromButton=True)
        if command['-i'] is None:
            showWarningMessageBox('No input file.', self)
            self.inFileLineEdit.selectAll()
            self.inFileLineEdit.setFocus()
            return False

        # Input file format
        inFormat = self.fileFormatCBox.currentText()

        # Chromosome name
        if self.chromNameLineEdit.text():
            command['-icn'] = self.chromNameLineEdit.text()

        # Dataset Title
        command['-t'] = '"' + self.titleLineEdit.text() + '"'

        # Resolutions list
        resolutions = self.checkResolutions()
        if resolutions is not None:
            command['-r'] = '"' + ', '.join(resolutions) + '"'

        # List of downsampling methods
        downsampling_methods= self.checkDownsamplingMethods()
        if downsampling_methods is None:
            return False
        else:
            command['-dm'] = '"' + ', '.join(downsampling_methods) + '"'

        # Scratch directory
        workDir = self.checkScratchDirectory(fromButton=True)
        if workDir is None:
            return False
        else:
            if workDir != config['Dirs']['WorkingDirectory']:
                command['-wd'] = '"{0}"'.format(workDir)

        # Output file
        command['-o'] = self.validateH5OutFile()
        if command['-o'] is None:
            return False

        # Compression method in output file
        command['-cmeth'] = self.compressionCBox.currentText().lower()

        # Path to bigwig programs
        if inFormat == 'bigWig':
            bigWigInfo = self.checkBigWigInfoProgram(fromButton=True)
            if bigWigInfo is None:
                return False
            else:
                if bigWigInfo != config['Programs']['bigWigInfo']:
                    command['-binfo'] = '"{0}"'.format(bigWigInfo)

            bigWigToWig = self.checkBigWigToWigProgram(fromButton=True)
            if bigWigToWig is None:
                return False
            else:
                if bigWigToWig != config['Programs']['bigWigToWig']:
                    command['-b2w'] = '"{0}"'.format(bigWigToWig)

        # Data column in bed file
        if inFormat == 'bed':
            command['-dtc'] = str( self.bedDataColumnSpinBox.value() )

        # Keep original data
        if self.keepOriginalButton.isChecked():
            command['-ko'] = ' '

        # Overwrite h5 dataset
        command['-ow'] = ' '

        # json Index file
        if self.indexFile is not None:
            command['-idf'] = '"{0}"'.format(self.indexFile)

        # Construct command
        commandText = ' '
        if inFormat == 'bigWig':
            commandText += 'bigwig2h5 '

        if inFormat == 'wig':
            commandText += 'wig2h5 '

        if inFormat == 'bed':
            commandText += 'bed2h5 '

        for key in command:
            commandText += key + ' ' + command[key] + ' '

        self.command = commandText

        return True

    def startProcess(self):
        if not self.buildCommand():
            return

        if self.command is None:
            return

        self.process = QProcess(self)
        self.hasConvertedSuccessfully = False

        self.process.start('gcMapExplorer', shlex.split(self.command))
        self.process.setProcessChannelMode( QProcess.MergedChannels )
        #self.process.setReadChannel( QProcess.StandardOutput )
        self.process.waitForStarted()

        self.process.readyReadStandardOutput.connect( self.writeLogOutputFromProcess )
        self.process.readyReadStandardError.connect( self.writeLogOutputFromProcess )
        self.process.finished.connect( self.finishedProcessing )

        self.convertButton.setEnabled(False)
        self.stopButton.setEnabled(True)
        self.logOutputPlainTextEdit.clear()

    def writeLogOutputFromProcess(self):
        """ Doverting process output to log box
        """
        if self.process is None:
            return

        out = bytes(self.process.readAllStandardOutput()).decode("utf-8")
        if out:
            out = out.rstrip()
            self.logOutputPlainTextEdit.appendPlainText( out  )
        out = bytes(self.process.readAllStandardError()).decode("utf-8")
        if out:
            out = out.rstrip()
            self.logOutputPlainTextEdit.appendPlainText( out  )

    def terminateProcessing(self):
        if self.process is None: return
        self.process.terminate()

    def finishedProcessing(self):
        self.convertButton.setEnabled(True)
        self.stopButton.setEnabled(False)
        exitStatus = self.process.exitStatus()
        self.process = None
        # Store result if process was normal exited
        if exitStatus == QProcess.NormalExit:
            self.hasConvertedSuccessfully = True
            # Hide automatically when process exited normally
            if self.hideMode:
                self.accept()
                self.disconnectFromAccepted()

    def connectToBrowserDataLoader(self, browser, gpa):
        self.accepted.connect( lambda : self.loadDataToBrowser(browser, gpa) )

    def disconnectFromAccepted(self):
        """ disconnect accpted
        """
        try:
            self.accepted.disconnect()
        except:
            pass

    def loadDataToBrowser(self, browser, gpa):
        """ This is used in the browser.
        Do not use in standalone mode.
        """

        if self.result() == QDialog.Accepted:
            # Open dialogbox so user can select a dataset
            gpa.selectGenomicDataHdf5ByDialogBox(self.h5FileLineEdit.text(), browser.filesOpened)
            self.centralOperationWidget.setEnabled( False )

        browser.loadDataToPlot(gpa)

    def connectoToBrowseMapChanger(self, browser, newMapName):
        """ Connect accepted to browser map changer
        """
        self.accepted.connect( lambda : browser.changeMapNames(newMapName) )

def showWarningMessageBox(msg, qwidget):
    msgBox = QMessageBox(QMessageBox.Warning, 'Warning', msg, QMessageBox.Ok, qwidget)
    msgBox.exec_()
    msgBox.close()

def checkDirExist(lineEdit, qwidget):
    if not lineEdit.text(): return
    dirname = str( lineEdit.text() )
    if not os.path.isdir( dirname ):
        msg = "[ {0} ] \n Not found !!!".format(dirname)
        msgBox = QMessageBox(QMessageBox.Warning, 'Warning', msg,
                                                    QMessageBox.Ok, qwidget)
        msgBox.exec_()
        msgBox.close()

        lineEdit.selectAll()
        lineEdit.setFocus()

def checkFileExist(lineEdit, qwidget):
    if not lineEdit.text(): return False
    filename = str( lineEdit.text() )
    if not os.path.isfile( filename ):
        msg = "[ {0} ] \n Not found !!!".format(filename)
        msgBox = QMessageBox(QMessageBox.Warning, 'Warning', msg,
                                                    QMessageBox.Ok, qwidget)
        msgBox.exec_()
        msgBox.close()

        lineEdit.selectAll()
        lineEdit.setFocus()

        return False
    else:
        return True

if __name__ == '__main__':
    main()



#!/usr/bin/env python3
#
# Author: Rajendra Kumar
#
# This file is part of gcMapExplorer
# Copyright (C) 2016-2017  Rajendra Kumar, Ludvig Lizana, Per Stenberg
#
# gcMapExplorer is a free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gcMapExplorer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gcMapExplorer.  If not, see <http://www.gnu.org/licenses/>.
#
#=============================================================================

import numpy as np
import tempfile
import logging

import scipy.sparse as sparse
# import scipy.sparse.linalg as sparse_linalg
from scipy.linalg import get_blas_funcs
from scipy import linalg as sp_linalg
import string, random
import os
import time

from . import ccmapHelpers as cmh
from . import ccmap as cmp
from . import gcmap as gmp
from . import normalizeKnightRuiz as krnorm
from . import normalizeAverageContact as avcnorm
from . import normalizeIC as icnorm

dtype_npBINarray = 'float32'

logger = logging.getLogger('normalizer')
logger.setLevel(logging.INFO)

def _checkCCMAP(ccMap, workDir=None):
	""" Check if ccmap is a obejct and file

	Returns
	-------
	ccMap : :class:`gcMapExplorer.lib.ccmap.CCMAP` or str
		CCMAP object or ccmap file.

	ccmapType : str
		'File' or Object

	"""
	# Check whether input is a file or a obejct
	ccmapType = 'Object'
	ccMapObj = None
	if not isinstance(ccMap, cmp.CCMAP):
		ccmapType = 'File'
		if not os.path.isfile(ccMap):
			logger.info(' {0} file not found.'.format(ccMap))
			logger.info(' Not able to normalize.')
			return None
		else:
			ccMapObj = cmp.load_ccmap(ccMap, workDir=workDir)
	else:
		ccMapObj = ccMap

	return ccMapObj, ccmapType

def NormalizeKnightRuizOriginal(ccMapObj, tol=1e-12, x0=None, delta=0.1, Delta=3, fl=0):
	'''Original Knight-Ruiz algorithm for matrix balancing

	Ported from a matlab script given in the supporting information of the following paper:
		* P.A. Knight and D. Ruiz (2013). A fast algorithm for matrix balancing (2013). IMA Journal of Numerical Analysis, 33, 1029-1047"
		* Matrix must be symmetric and non-negative
		* For input matrix A, this function find a vector X such that diag(X)*A*diag(X) is close to doubly stochastic.

	.. warning::
		* This is original ported code and kept here for comparison and testing.
		* Do not use it because for large matrix it may end up with consuming all the memory for large matrix.

	Parameters
	----------
	ccMapObj : :class:`gcMapExplorer.lib.ccmap.CCMAP`
		A CCMAP object containing observed contact frequency

	Returns
	-------
	normCCMap : :class:`CCMAP`
		Normalized Contact map.

	'''

	normCCMap = ccMapObj.copy()
	normCCMap.make_editable()
	ccMapObj.make_readable()

	normCCMap.bNoData = np.all( ccMapObj.matrix == 0.0, axis=0)
	bNonZeros = ~normCCMap.bNoData
	A = (ccMapObj.matrix[bNonZeros,:])[:,bNonZeros]   # Selected row-column which are not all zeros

	BinNormMatrix = np.memmap(normCCMap.path2matrix, dtype=dtype_npBINarray, mode='w+', shape=normCCMap.shape)

	n = A.shape[0]                                                                 # n = size(A,1)
	e = np.ones((n, 1))                                                                         # e = ones(n,1)
	res = []
	if x0 is None:
		x0 = e


	g = 0.9        # Parameters used in inner stopping criterion.
	etamax = 0.1   # Parameters used in inner stopping criterion.
	eta = etamax
	stop_tol = tol * 0.5
	x = x0
	rt = tol**2                                 # rt = tol^2
	v = x * A.dot(x)                            # v = x.*(A*x)
	rk = 1 - v
	rho_km1 = np.dot( rk.conjugate().T, rk)     # rho_km1 = rk'*rk
	rout = rho_km1
	rold = rout

	# x, x0, e, v, rk, y, Z, w, p, ap :     vector shape(n, 1) : [ [value] [value] [value] [value] ... ... ... [value] ]
	# rho_km1, rout, rold, innertol, alpha :  scalar shape(1 ,1) : [[value]]

	MVP = 0                                                      # Well count matrix vector products.
	i = 0                                                        # Outer iteration count.

	if fl == 1:
		print('it in. it res')

	while rout > rt: 										     # Outer iteration
		i = i + 1
		k = 0
		y = e
		innertol = max( [ eta**2 * rout, rt ])                   # innertol = max([eta^2*rout,rt]);

		while rho_km1 > innertol:                                # Inner iteration by CG
			k = k + 1
			if k == 1:
				Z = rk / v                                       # Z = rk./v
				p = Z
				rho_km1 = np.dot( rk.conjugate().T, Z)           # rho_km1 = rk'*Z
			else:
				beta = rho_km1 / rho_km2
				p = Z + (beta * p)


			# Update search direction efficiently.
			w = x * A.dot((x*p)) + (v * p)                       # w = x.*(A*(x.*p)) + v.*p
			alpha = rho_km1 / np.dot( p.conjugate().T, w)        # alpha = rho_km1/(p'*w)
			ap = alpha * p                                       # ap = alpha*p (No dot function as alpha is scalar)

			# Test distance to boundary of cone.
			ynew = y + ap;
			#print(i, np.amin(ynew), delta, np.amin(ynew) <= delta)
			#print(i, np.amax(ynew), Delta, np.amax(ynew) >= Delta)
			if np.amin(ynew) <= delta:
				if delta == 0:
					break
				ind = np.nonzero(ap < 0) # ind = find(ap < 0)
				gamma = np.amin( (delta - y[ind]) / ap[ind] )    # gamma = min((delta - y(ind))./ap(ind))
				y = y + np.dot(gamma, ap)                        # y = y + gamma*ap
				break
			if np.amax(ynew) >= Delta:
				ind = np.nonzero( ynew > Delta )                 # ind = find(ynew > Delta);
				gamma = np.amin( (Delta-y[ind]) / ap[ind])       # gamma = min((Delta-y(ind))./ap(ind));
				y = y + np.dot(gamma, ap)                        # y = y + gamma*ap;
				break
			y = ynew
			rk = rk - alpha*w                                    # rk = rk - alpha*w
			rho_km2 = rho_km1
			Z = rk / v
			rho_km1 = np.dot( rk.conjugate().T, Z)               # rho_km1 = rk'*Z


		x = x * y                                                # x = x.*y
		v = x * A.dot(x)                                         # v = x.*(A*x)
		rk = 1 - v
		rho_km1 = np.dot( rk.conjugate().T, rk)                  # rho_km1 = rk'*rk
		rout = rho_km1
		MVP = MVP + k + 1

		# Update inner iteration stopping criterion.
		rat = rout/rold
		rold = rout
		res_norm = np.sqrt(rout)
		eta_o = eta
		eta = g*rat

		#print(i, res_norm)

		if g*eta_o**2 > 0.1:
			eta = np.amax([eta, g*eta_o**2])                    # eta = max([eta,g*eta_o^2])

		eta = np.amax([np.amin([eta, etamax]), stop_tol/res_norm]);   # eta = max([min([eta,etamax]),stop_tol/res_norm]);

		if fl == 1:
			print('%3d %6d %.3e %.3e %.3e \n' % (i,k, res_norm, np.amin(y), np.amin(x)))
			res=[res, res_norm]


	# Generation of Doubly stochastic matrix ( diag(X)*A*diag(X) )
	(fd, path2matrix) = tempfile.mkstemp(suffix='.bin', prefix='nparray_', dir=None, text=False)
	os.close(fd)     # Close file, error in windows OS
	A_DSMat = np.memmap(path2matrix, dtype=dtype_npBINarray, mode='w+', shape=A.shape)
	A_DSMat[:] = x.T * (A * x)

	# Assigning minvalue and maxvalue
	normCCMap.maxvalue = np.float(np.amax(A_DSMat))
	minvalue = np.amin(A_DSMat)
	v_steps = np.linspace(minvalue, normCCMap.maxvalue, 100)
	normCCMap.minvalue = minvalue - (v_steps[1] - v_steps[0])

	if (normCCMap.minvalue < 0.0):
		normCCMap.minvalue = 0.0

	dsm_i = -1
	dsm_j = 0
	for i in range(BinNormMatrix.shape[0]):
		if not normCCMap.bNoData[i]:
			dsm_i += 1

		dsm_j = 0
		for j in range(BinNormMatrix.shape[1]):
			if normCCMap.bNoData[i] or normCCMap.bNoData[j]:
				BinNormMatrix[i][j] = normCCMap.minvalue
			else:
				BinNormMatrix[i][j] = A_DSMat[dsm_i][dsm_j]
				dsm_j += 1


	# To check if sum of rows and columns are one
	r_sum = A_DSMat.sum(axis = 0)
	c_sum = A_DSMat.sum(axis = 1)
	#for i in range(A_DSMat.shape[0]):
		#print(i, x[i], r_sum[i], c_sum[i])


	BinNormMatrix.flush()
	del BinNormMatrix
	del A_DSMat

	try:
		os.remove(path2matrix)
	except:
		pass

	return normCCMap

def normalizeCCMapByKR(ccMap, memory='RAM', tol=1e-12, outFile=None, vmin=None, vmax=None, percentile_thershold_no_data=None, thershold_data_occup=None, workDir=None):
	"""Normalize a ccmap using Knight-Ruiz matrix balancing method.

	.. note::
		* This function uses a modified version of orginal ported code given in :meth:`NormalizeKnightRuizOriginal`.
		* **Please refer to:** P.A. Knight and D. Ruiz (2013). A fast algorithm for matrix balancing (2013). IMA Journal of Numerical Analysis, 33, 1029-1047

	Parameters
	----------
	ccMap : :class:`gcMapExplorer.lib.ccmap.CCMAP` or ccmap file
		A CCMAP object containing observed contact frequency or a ccmap file.

	memory : str
		Accepted keywords are ``RAM`` and ``HDD``:

			* ``RAM``: All intermediate arrays are generated in memory(RAM). This version is faster, however, it requires RAM depending on the input matrix size.
			* ``HDD``: All intermediate arrays are generated as memory mapped array files on hard-disk.

	tol : float
		Tolerance for matrix balancing. Smaller tolreance increases accuracy in sums of rows and columns.

	outFile : str
		Name of output ccmap file, to save directly the normalized map as a ccmap file. In case of this option, ``None`` will return.

	vmin : float
		Minimum thershold value for normalization. If contact frequency is less than or equal to this thershold value, this value is discarded during normalization.

	vmax : float
		Maximum thershold value for normalization. If contact frequency is greater than or equal to this thershold value, this value is discarded during normalization.

	percentile_thershold_no_data : int
		It can be used to filter the map, where rows/columns with largest numbers of missing data can be discarded.
		``percentile_thershold_no_data`` should be between 1 and 100. This options discard the rows and columns which are above this percentile.
		For example: if this value is 99, those row or columns will be discarded which contains larger than number of zeros (missing data) at 99 percentile.

		To calculate percentile, all blank rows are removed, then in all rows, number of zeros are counted. Afterwards, number of zeros at
		`percentile_thershold_no_data` percentile is obtained. In next step, if a row contain number of zeros larger than this percentile value,
		the whole row and column is assigned to have missing data. This percentile indicates highest numbers of zeros (missing data) in given rows/columns.

	thershold_data_occup : float
		It can be used to filter the map, where rows/columns with largest numbers of missing data can be discarded.
		This ratio is (number of bins with data) / (total number of bins in the given row/column).
		For example: if `thershold_data_occup = 0.8`, then all rows containing more than 20\% of missing data will be discarded.

		Note that this parameter is suitable for low resolution data because maps are likely to be much less sparse.

	workDir : str
		Path to the directory where temporary intermediate files are generated.
		If ``None``, files are generated in the temporary directory according to the main configuration.


	Returns
	-------
	ccMapObj : :class:`gcMapExplorer.lib.ccmap.CCMAP` or ``None``
		Normalized Contact map. When ``outFile`` is provided, ``None`` is returned. In case of any other error, ``None`` is returned.

	"""

	# Check whether input is a file or a obejct
	ccMapObjOrig, ccmapType = _checkCCMAP(ccMap, workDir=workDir)

	# Make another copy here for maximum and minimum thershold value
	if vmin is not None or vmax is not None:
		ccMapObj = ccMapObjOrig.copy()
		ccMapObj.make_editable()

		if ccmapType == 'File':
			del ccMapObjOrig

		ccmapType = 'File' # This temporary file should be deleted when neccessary
		if vmin is not None:
			ccMapObj.matrix[ np.nonzero(ccMapObj.matrix <= vmin) ] = 0.0
		if vmax is not None:
			ccMapObj.matrix[ np.nonzero(ccMapObj.matrix >= vmax) ] = 0.0
		ccMapObj.matrix.flush()
		ccMapObj.make_unreadable()

	else:
		ccMapObj = ccMapObjOrig

	normCCMap = ccMapObj.copy(fill=0.0)
	normCCMap.make_editable()
	ccMapObj.make_readable()

	logger.info(' KR Normalization will be done through {0}.'.format(memory))
	if ccMapObj.xlabel is not None:
		logger.info(' KR Normalization is in process for {0} map...'.format(ccMapObj.xlabel))
	else:
		logger.info(' KR Normalization is in process for UNKNOWN map!!')

	# Default Parameters
	delta=0.1
	Delta=3
	fl=0

	# Main section
	try:

		if memory=='RAM':
			bNonZeros = cmh.get_nonzeros_index(ccMapObj.matrix, thershold_percentile=percentile_thershold_no_data, thershold_data_occup=thershold_data_occup)

			A = (ccMapObj.matrix[bNonZeros,:])[:,bNonZeros]   # Selected row-column which are not all zeros

			normCCMap.bNoData = ~bNonZeros
			del bNonZeros

			# Added one so that all value with zero will be one
			KRObj = krnorm.KnightRuizNorm(A, tol=tol, delta=delta, Delta=Delta, workDir=workDir)
			minvalue, maxvalue = KRObj.run(A, fl, normCCMap.matrix, normCCMap.bNoData)

		elif memory == 'HDD':

			# Selected row-column which are not all zeros
			A, normCCMap.bNoData  = cmh.remove_zeros(ccMapObj.matrix, thershold_percentile=percentile_thershold_no_data,
			                                            thershold_data_occup=thershold_data_occup, workDir=workDir)

			# Try for removing temporary files related to above variable A
			try:
				KRObj = krnorm.KnightRuizNorm(A.arr, memory='HDD', tol=tol, delta=delta, Delta=Delta, workDir=workDir)
				minvalue, maxvalue = KRObj.run(A.arr, fl, normCCMap.matrix, normCCMap.bNoData)
			except(KeyboardInterrupt, SystemExit) as e:
				del A
				raise e
			del A

		else:
			raise ValueError ('This memory={0} is not is not understandable.. Please use \'RAM\' or \'HDD\'.' .format(memory))

		normCCMap.maxvalue = float(maxvalue)
		normCCMap.minvalue = float(minvalue)

		normCCMap.make_unreadable()
		ccMapObj.make_unreadable()

		if ccMapObj.xlabel is not None:
			logger.info(' 	...Finished KR Normalization for {0} map...'.format(ccMapObj.xlabel))
		else:
			logger.info(' 	...Finished KR Normalization for UNKNOWN map!!')


		# When outFile is provided write a output
		if outFile is not None:
			cmp.save_ccmap(normCCMap, outFile, compress=True)

		# Delete ccmap object if input was a file
		if ccmapType == 'File':
			del ccMapObj

		# Whether outFile is given.
		if outFile is None:
			return normCCMap
		else:
			del normCCMap
			return None

	# In case of program termination, delete the newly created ccmap and raise error
	except (KeyboardInterrupt, SystemExit) as e:
		if 'normCCMap' in locals():	del normCCMap
		if ccmapType == 'File' and 'ccMapObj' in locals():	del ccMapObj
		raise e

	# In case of other error, delete the newly created ccmap, print error and return None
	except Exception as e:
		if 'normCCMap' in locals():	del normCCMap
		if ccmapType == 'File' and 'ccMapObj' in locals():	del ccMapObj
		logger.warning(e)
		logger.warning('Error in normalizing map!!!')
		return None

def normalizeGCMapByKR(gcMapInputFile, gcMapOutFile, mapSizeCeilingForMemory=20000, vmin=None, vmax=None, tol=1e-12, percentile_thershold_no_data=None, thershold_data_occup=None, compression='lzf', workDir=None, logHandler=None):
	"""Normalize a gcmap using Knight-Ruiz matrix balancing method.


	Parameters
	----------
	gcMapInputFile : str
		Name of input gcmap file.

	gcMapOutFile : str
		Name of output gcmap file.

	mapSizeCeilingForMemory : int
		Maximum size of contact map allowed for calculation using RAM. If map size or shape is larger than this value,
		normalization will be performed using disk (HDD).

	vmin : float
		Minimum thershold value for normalization. If contact frequency is less than or equal to this thershold value, this value is discarded during normalization.

	vmax : float
		Maximum thershold value for normalization. If contact frequency is greater than or equal to this thershold value, this value is discarded during normalization.

	tol : float
		Tolerance for matrix balancing. Smaller tolreance increases accuracy in sums of rows and columns.

	percentile_thershold_no_data : int
		It can be used to filter the map, where rows/columns with largest numbers of missing data can be discarded.
		``percentile_thershold_no_data`` should be between 1 and 100. This options discard the rows and columns which are above this percentile.
		For example: if this value is 99, those row or columns will be discarded which contains larger than number of zeros (missing data) at 99 percentile.

		To calculate percentile, all blank rows are removed, then in all rows, number of zeros are counted. Afterwards, number of zeros at
		`percentile_thershold_no_data` percentile is obtained. In next step, if a row contain number of zeros larger than this percentile value,
		the whole row and column is assigned to have missing data. This percentile indicates highest numbers of zeros (missing data) in given rows/columns.

	thershold_data_occup : float
		It can be used to filter the map, where rows/columns with largest numbers of missing data can be discarded.
		This ratio is (number of bins with data) / (total number of bins in the given row/column).
		For example: if `thershold_data_occup = 0.8`, then all rows containing more than 20\% of missing data will be discarded.

		Note that this parameter is suitable for low resolution data because maps are likely to be much less sparse.

	compression : str
	    Compression method in output gcmap file. Presently allowed : ``lzf`` for LZF compression and ``gzip`` for GZIP compression.

	workDir : str
		Path to the directory where temporary intermediate files are generated.
		If ``None``, files are generated in the temporary directory according to the main configuration.


	Returns
	-------
	None


	.. seealso::
		:meth:`gcMapExplorer.lib.normalizer.normalizeCCMapByKR`

	"""

	# Get list of maps in ascending order
	gcmap = gmp.GCMAP(gcMapInputFile)
	gcmap.loadSmallestMap()
	mapList = gcmap.mapNameList.copy()
	del gcmap

	for mapName in mapList:
		ccMap = gmp.loadGCMapAsCCMap(gcMapInputFile, mapName=mapName, workDir=workDir)

		# Because ccMap is already loaded here, directly edit matrix here
		# No need to pass vmin and vmx during normalization as
		# it is already taken care here
		if vmin is not None or vmax is not None:
			ccMap.make_editable()
			if vmin is not None:
				ccMap.matrix[ np.nonzero(ccMap.matrix <= vmin) ] = 0.0
			if vmax is not None:
				ccMap.matrix[ np.nonzero(ccMap.matrix >= vmax) ] = 0.0
			ccMap.matrix.flush()
			ccMap.make_unreadable()

		try:

			if ccMap.shape[0] > mapSizeCeilingForMemory:
				memory = 'HDD'
			else:
				memory = 'RAM'

			norm_ccmap = normalizeCCMapByKR(ccMap, memory=memory, tol=tol,
						percentile_thershold_no_data=percentile_thershold_no_data,
						thershold_data_occup=thershold_data_occup,
						workDir=workDir)

			if norm_ccmap is not None:
				gmp.addCCMap2GCMap(norm_ccmap, gcMapOutFile,
									compression=compression,
									generateCoarse=True, coarsingMethod='sum',
									logHandler=logHandler)

		# In case of program termination, delete the newly created ccmap and raise error
		except (KeyboardInterrupt, SystemExit) as e:
			if 'ccMap' in locals():	del ccMap
			raise e

def normalizeCCMapByIC(ccMap, tol=1e-4, vmin=None, vmax=None, outFile=None, iteration=500, percentile_thershold_no_data=None, thershold_data_occup=None, workDir=None):
	""" Normalize a ccmap by Iterative correction method

	This method normalize the raw contact map by removing biases from experimental procedure.
	For more details, see `this publication <http://www.nature.com/nmeth/journal/v9/n10/full/nmeth.2148.html>`_.

	Parameters
	----------
	ccMap : :class:`gcMapExplorer.lib.ccmap.CCMAP` or ccmap file.
		A CCMAP object containing observed contact frequency or a ccmap file

	tol : float
		Tolerance value. If variance of Delta-B is less than tolerance, stop the iterative process.

	vmin : float
		Minimum thershold value for normalization. If contact frequency is less than or equal to this thershold value, this value is discarded during normalization.

	vmax : float
		Maximum thershold value for normalization. If contact frequency is greater than or equal to this thershold value, this value is discarded during normalization.

	outFile : str
		Name of output ccmap file, to save directly the normalized map as a ccmap file. In case of this option, ``None`` will return.

	iteration : int
		Number of iteration to stop the normalization.

	percentile_thershold_no_data : int
		It can be used to filter the map, where rows/columns with largest numbers of missing data can be discarded.
		``percentile_thershold_no_data`` should be between 1 and 100. This options discard the rows and columns which are above this percentile.
		For example: if this value is 99, those row or columns will be discarded which contains larger than number of zeros (missing data) at 99 percentile.

		To calculate percentile, all blank rows are removed, then in all rows, number of zeros are counted. Afterwards, number of zeros at
		`percentile_thershold_no_data` percentile is obtained. In next step, if a row contain number of zeros larger than this percentile value,
		the whole row and column is assigned to have missing data. This percentile indicates highest numbers of zeros (missing data) in given rows/columns.

	thershold_data_occup : float
		It can be used to filter the map, where rows/columns with largest numbers of missing data can be discarded.
		This ratio is (number of bins with data) / (total number of bins in the given row/column).
		For example: if `thershold_data_occup = 0.8`, then all rows containing more than 20\% of missing data will be discarded.

		Note that this parameter is suitable for low resolution data because maps are likely to be much less sparse.

	workDir : str
		Path to the directory where temporary intermediate files are generated.
		If ``None``, files are generated in the temporary directory according to the main configuration.

	Returns
	-------
	normCCMap : :class:`gcMapExplorer.lib.ccmap.CCMAP` or ``None``
		Normalized Contact map. When ``outFile`` is provided, ``None`` is returned. In case of any other error, ``None`` is returned.

	"""

	# Check whether input is a file or a obejct
	ccMapObj, ccmapType = _checkCCMAP(ccMap, workDir=workDir)

	tmap = ccMapObj.copy()

	# In case if vmin and vmax is given
	if vmin is not None or vmax is not None:
		tmap.make_editable()
		if vmin is not None:
			tmap.matrix[ np.nonzero(tmap.matrix <= vmin) ] = 0.0
		if vmax is not None:
			tmap.matrix[ np.nonzero(tmap.matrix >= vmax) ] = 0.0
		tmap.make_unreadable()

	if ccMapObj.xlabel is not None:
		logger.info(' Iterative Correction is in process for {0} map...'.format(ccMapObj.xlabel))
	else:
		logger.info(' Iterative Correction is in process for UNKNOWN map!!')

	# Check if any filter is applied
	applyFilter = True
	if percentile_thershold_no_data is None and thershold_data_occup is None:
	    applyFilter = False

	try:
		tmap.make_editable()

		matrix = None
		bNonZeros = None
		if applyFilter:
		    bNonZeros = cmh.get_nonzeros_index(tmap.matrix, thershold_percentile=percentile_thershold_no_data, thershold_data_occup=thershold_data_occup)
		    matrix = (tmap.matrix[bNonZeros,:])[:,bNonZeros]   # Selected row-column which are not all zeros
		    icnorm.performIterativeCorrection(matrix, tol, iteration)
		else:
			matrix = tmap.matrix
			icnorm.performIterativeCorrection(matrix, tol, iteration)
			bNonZeros = ~np.all( tmap.matrix == 0.0, axis=0)

		ma = np.ma.masked_equal(matrix, 0.0, copy=False)
		tmap.minvalue = ma.min()
		tmap.maxvalue = ma.max()
		tmap.bNoData = ~bNonZeros

		if applyFilter:
		    # Fill original output matrix
		    dsm_i = 0
		    ox = tmap.matrix.shape[0]
		    idx_fill = np.nonzero( bNonZeros )
		    for i in range(ox):
		        if not tmap.bNoData[i]:
		            tmap.matrix[i, idx_fill] = matrix[dsm_i]
		            tmap.matrix[idx_fill, i] = matrix[dsm_i]
		            dsm_i += 1
		        else:
		            idx_nonzero = np.nonzero( tmap.matrix[i] > 0 )
		            tmap.matrix[i][idx_nonzero].fill(0.0)

		if ccMapObj.xlabel is not None:
			logger.info(' 	...Finished Iterative Correction for {0} map...'.format(ccMapObj.xlabel))
		else:
			logger.info(' 	...Finished Iterative Correction for UNKNOWN map!!')

		# Save output ccmap file
		if outFile is not None:
			cmp.save_ccmap(tmap, outFile, compress=True)

		# Delete ccmap object if input was a file
		if ccmapType == 'File':
			del ccMapObj

		# Whether outFile is given.
		if outFile is None:
			return tmap
		else:
			del tmap
			return None

	# In case of program termination, delete the newly created ccmap and raise error
	except (SystemExit, KeyboardInterrupt) as e:
		if 'tmap' in locals():	del tmap
		if ccmapType == 'File' and 'ccMapObj' in locals():	del ccMapObj
		raise e

	# In case of other error, delete the newly created ccmap, print error and return None
	except Exception as e:
		if 'tmap' in locals():	del tmap
		if ccmapType == 'File' and 'ccMapObj' in locals():	del ccMapObj
		logger.warning(e)
		logger.warning('Error in Iterative Correction!!!')
		return None

def normalizeGCMapByIC(gcMapInputFile, gcMapOutFile, vmin=None, vmax=None, tol=1e-12, iteration=500, percentile_thershold_no_data=None, thershold_data_occup=None, compression='lzf', workDir=None, logHandler=None):
	"""Normalize a gcmap using Iterative Correction.

	This method normalize the raw contact map by removing biases from experimental procedure.
	For more details, see `this publication <http://www.nature.com/nmeth/journal/v9/n10/full/nmeth.2148.html>`_.

	Parameters
	----------
	gcMapInputFile : str
		Name of input gcmap file.

	gcMapOutFile : str
		Name of output gcmap file.

	vmin : float
		Minimum thershold value for normalization. If contact frequency is less than or equal to this thershold value, this value is discarded during normalization.

	vmax : float
		Maximum thershold value for normalization. If contact frequency is greater than or equal to this thershold value, this value is discarded during normalization.

	tol : float
		Tolerance value. If variance of Delta-B is less than tolerance, stop the iterative process.

	iteration : int
		Number of iteration to stop the normalization.

	percentile_thershold_no_data : int
		It can be used to filter the map, where rows/columns with largest numbers of missing data can be discarded.
		``percentile_thershold_no_data`` should be between 1 and 100. This options discard the rows and columns which are above this percentile.
		For example: if this value is 99, those row or columns will be discarded which contains larger than number of zeros (missing data) at 99 percentile.

		To calculate percentile, all blank rows are removed, then in all rows, number of zeros are counted. Afterwards, number of zeros at
		`percentile_thershold_no_data` percentile is obtained. In next step, if a row contain number of zeros larger than this percentile value,
		the whole row and column is assigned to have missing data. This percentile indicates highest numbers of zeros (missing data) in given rows/columns.

	thershold_data_occup : float
		It can be used to filter the map, where rows/columns with largest numbers of missing data can be discarded.
		This ratio is (number of bins with data) / (total number of bins in the given row/column).
		For example: if `thershold_data_occup = 0.8`, then all rows containing more than 20\% of missing data will be discarded.

		Note that this parameter is suitable for low resolution data because maps are likely to be much less sparse.

	compression : str
	    Compression method in output gcmap file. Presently allowed : ``lzf`` for LZF compression and ``gzip`` for GZIP compression.

	workDir : str
		Path to the directory where temporary intermediate files are generated.
		If ``None``, files are generated in the temporary directory according to the main configuration.


	Returns
	-------
	None


	.. seealso::
		:meth:`gcMapExplorer.lib.normalizer.normalizeCCMapByIC`


	"""

	# Get list of maps in ascending order
	gcmap = gmp.GCMAP(gcMapInputFile)
	gcmap.loadSmallestMap()
	mapList = gcmap.mapNameList.copy()
	del gcmap

	for mapName in mapList:
		ccMap = gmp.loadGCMapAsCCMap(gcMapInputFile, mapName=mapName, workDir=workDir)

		try:
			norm_ccmap = normalizeCCMapByIC(ccMap, tol=tol, iteration=iteration,
						vmin=vmin, vmax=vmax,
						percentile_thershold_no_data=percentile_thershold_no_data,
						thershold_data_occup=thershold_data_occup,
						workDir=workDir)

			if norm_ccmap is not None:
				gmp.addCCMap2GCMap(norm_ccmap, gcMapOutFile,
									compression=compression,
									generateCoarse=True, coarsingMethod='sum',
									logHandler=logHandler)

		# In case of program termination, delete the newly created ccmap and raise error
		except (KeyboardInterrupt, SystemExit) as e:
			if 'ccMap' in locals():	del ccMap
			raise e

def normalizeCCMapByMCFS(ccMap, stats='median', vmin=None, vmax=None, outFile=None, percentile_thershold_no_data=None, thershold_data_occup=None, workDir=None):
	""" Scale ccmap using Median Contact Frequency

	This method can be used to normalize contact map using Median contact values
	for particular distance between two locations/coordinates. At first, Median
	distance contact frequency for each distance is calculated. Subsequently,
	the observed contact frequency is divided by median contact frequency
	obtained for distance between the two locations.

	.. note:
		In place of median, mean can be also used for scaling. See below for options.

	Parameters
	----------
	ccMapObj : :class:`gcMapExplorer.lib.ccmap.CCMAP` or ccmap file
		A CCMAP object containing observed contact frequency or a ccmap file

	stats : str
	    Statistics to be calculated along diagonals: It may be either "mean" or "median". By default, it is "median".

	vmin : float
		Minimum thershold value for normalization. If contact frequency is less than or equal to this thershold value, this value is discarded during normalization.

	vmax : float
		Maximum thershold value for normalization. If contact frequency is greater than or equal to this thershold value, this value is discarded during normalization.

	outFile : str
		Name of output ccmap file, to save directly the normalized map as a ccmap file. In case of this option, ``None`` will return.

	percentile_thershold_no_data : int
		It can be used to filter the map, where rows/columns with largest numbers of missing data can be discarded.
		``percentile_thershold_no_data`` should be between 1 and 100. This options discard the rows and columns which are above this percentile.
		For example: if this value is 99, those row or columns will be discarded which contains larger than number of zeros (missing data) at 99 percentile.

		To calculate percentile, all blank rows are removed, then in all rows, number of zeros are counted. Afterwards, number of zeros at
		`percentile_thershold_no_data` percentile is obtained. In next step, if a row contain number of zeros larger than this percentile value,
		the whole row and column is assigned to have missing data. This percentile indicates highest numbers of zeros (missing data) in given rows/columns.

	thershold_data_occup : float
		It can be used to filter the map, where rows/columns with largest numbers of missing data can be discarded.
		This ratio is (number of bins with data) / (total number of bins in the given row/column).
		For example: if `thershold_data_occup = 0.8`, then all rows containing more than 20\% of missing data will be discarded.

		Note that this parameter is suitable for low resolution data because maps are likely to be much less sparse.

	workDir : str
		Path to the directory where temporary intermediate files are generated.
		If ``None``, files are generated in the temporary directory according to the main configuration.

	Returns
	-------
	ccMapObj : :class:`gcMapExplorer.lib.ccmap.CCMAP` or ``None``
		Normalized Contact map. When ``outFile`` is provided, ``None`` is returned. In case of any other error, ``None`` is returned.

	"""

	# Check whether input is a file or a obejct
	ccMapObjOrig, ccmapType = _checkCCMAP(ccMap, workDir=workDir)

	# Make another copy here for maximum and minimum thershold value
	if vmin is not None or vmax is not None:
		ccMapObj = ccMapObjOrig.copy()
		ccMapObj.make_editable()

		if ccmapType == 'File':
			del ccMapObjOrig

		ccmapType = 'File' # This temporary file should be deleted when neccessary
		if vmin is not None:
			ccMapObj.matrix[ np.nonzero(ccMapObj.matrix <= vmin) ] = 0.0
		if vmax is not None:
			ccMapObj.matrix[ np.nonzero(ccMapObj.matrix >= vmax) ] = 0.0
		ccMapObj.matrix.flush()
		ccMapObj.make_unreadable()

	else:
		ccMapObj = ccMapObjOrig

	normCCMap = ccMapObj.copy()

	if ccMapObj.xlabel is not None:
		logger.info(' Median Contact Frequency Scaling is in process for {0} map...'.format(ccMapObj.xlabel))
	else:
		logger.info(' Median Contact Frequency Scaling is in process for UNKNOWN map!!')

	# Main section
	try:
		avcnorm._normalizeByAvgContact(ccMapObj, normCCMap, stats=stats,
					percentile_thershold_no_data=percentile_thershold_no_data,
					thershold_data_occup=thershold_data_occup)

		if ccMapObj.xlabel is not None:
			logger.info(' 	...Finished Median Contact Frequency Scaling for {0} map...'.format(ccMapObj.xlabel))
		else:
			logger.info(' 	...Finished Median Contact Frequency Scaling for UNKNOWN map!!')

		# Save output ccmap file
		if outFile is not None:
			cmp.save_ccmap(normCCMap, outFile, compress=True)

		# Delete ccmap object if input was a file
		if ccmapType == 'File':
			del ccMapObj

		# Whether outFile is given.
		if outFile is None:
			return normCCMap
		else:
			del normCCMap
			return None

	# In case of program termination, delete the newly created ccmap and raise error
	except (KeyboardInterrupt, SystemExit) as e:
		if 'normCCMap' in locals():	del normCCMap
		if ccmapType == 'File' and 'ccMapObj' in locals():	del ccMapObj
		raise e

	# In case of other error, delete the newly created ccmap, print error and return None
	except Exception as e:
		if 'normCCMap' in locals():	del normCCMap
		if ccmapType == 'File' and 'ccMapObj' in locals():	del ccMapObj
		logger.warning(e)
		logger.warning('Error in Median Contact Frequency Scaling!!!')
		return None

def normalizeGCMapByMCFS(gcMapInputFile, gcMapOutFile, stats='median', vmin=None, vmax=None, percentile_thershold_no_data=None, thershold_data_occup=None, compression='lzf', workDir=None, logHandler=None):
	""" Scale all maps in gcmap using Median Contact Frequency

	This method can be used to normalize contact map using Median contact values
	for particular distance between two locations/coordinates. At first, Median
	distance contact frequency for each distance is calculated. Subsequently,
	the observed contact frequency is divided by median contact frequency
	obtained for distance between the two locations.

	Parameters
	----------
	gcMapInputFile : str
		Name of input gcmap file.

	gcMapOutFile : str
		Name of output gcmap file.

	stats : str
	    Statistics to be calculated along diagonals: It may be either "mean" or "median". By default, it is "median".

	vmin : float
		Minimum thershold value for normalization. If contact frequency is less than or equal to this thershold value, this value is discarded during normalization.

	vmax : float
		Maximum thershold value for normalization. If contact frequency is greater than or equal to this thershold value, this value is discarded during normalization.

	percentile_thershold_no_data : int
		It can be used to filter the map, where rows/columns with largest numbers of missing data can be discarded.
		``percentile_thershold_no_data`` should be between 1 and 100. This options discard the rows and columns which are above this percentile.
		For example: if this value is 99, those row or columns will be discarded which contains larger than number of zeros (missing data) at 99 percentile.

		To calculate percentile, all blank rows are removed, then in all rows, number of zeros are counted. Afterwards, number of zeros at
		`percentile_thershold_no_data` percentile is obtained. In next step, if a row contain number of zeros larger than this percentile value,
		the whole row and column is assigned to have missing data. This percentile indicates highest numbers of zeros (missing data) in given rows/columns.

	thershold_data_occup : float
		It can be used to filter the map, where rows/columns with largest numbers of missing data can be discarded.
		This ratio is (number of bins with data) / (total number of bins in the given row/column).
		For example: if `thershold_data_occup = 0.8`, then all rows containing more than 20\% of missing data will be discarded.

		Note that this parameter is suitable for low resolution data because maps are likely to be much less sparse.

	compression : str
	    Compression method in output gcmap file. Presently allowed : ``lzf`` for LZF compression and ``gzip`` for GZIP compression.

	workDir : str
		Path to the directory where temporary intermediate files are generated.
		If ``None``, files are generated in the temporary directory according to the main configuration.


	Returns
	-------
	None


	.. seealso::
		:meth:`gcMapExplorer.lib.normalizer.normalizeCCMapByMCFS`


	"""

	# Get list of maps in ascending order
	gcmap = gmp.GCMAP(gcMapInputFile)
	gcmap.loadSmallestMap()
	mapList = gcmap.mapNameList.copy()

	for mapName in mapList:

		# Iterate over available resolutions
		gcmap.changeMap(mapName)
		previousResolution = gcmap.resolution

		while True:
			ccMap = gmp.loadGCMapAsCCMap(gcmap.hdf5, mapName=mapName, resolution=gcmap.resolution, workDir=workDir)

			# Because ccMap is already loaded here, directly edit matrix here
			# No need to pass vmin and vmx during normalization as
			# it is already taken care here
			if vmin is not None or vmax is not None:
				ccMap.make_editable()
				if vmin is not None:
					ccMap.matrix[ np.nonzero(ccMap.matrix <= vmin) ] = 0.0
				if vmax is not None:
					ccMap.matrix[ np.nonzero(ccMap.matrix >= vmax) ] = 0.0
				ccMap.matrix.flush()
				ccMap.make_unreadable()

			try:
				norm_ccmap = normalizeCCMapByMCFS(ccMap, stats=stats,
							percentile_thershold_no_data=percentile_thershold_no_data,
							thershold_data_occup=thershold_data_occup,
							workDir=workDir)

				if norm_ccmap is not None:
					gmp.addCCMap2GCMap(norm_ccmap, gcMapOutFile,
										compression=compression,
										generateCoarse=False, replaceCMap=False,
										logHandler=logHandler)

					del norm_ccmap

				else:
					break

				gcmap.toCoarserResolution()
				if previousResolution == gcmap.resolution:
					break
				else:
					previousResolution = gcmap.resolution

			# In case of program termination, delete the newly created ccmap and raise error
			except (KeyboardInterrupt, SystemExit) as e:
				if 'norm_ccmap' in locals():	del norm_ccmap
				if 'gcmap' in locals():	del gcmap
				raise e



#!/usr/bin/env python3
#
# Author: Rajendra Kumar
#
# This file is part of gcMapExplorer
# Copyright (C) 2016-2017  Rajendra Kumar, Ludvig Lizana, Per Stenberg
#
# gcMapExplorer is a free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gcMapExplorer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gcMapExplorer.  If not, see <http://www.gnu.org/licenses/>.
#
#=============================================================================

import os
import sys
import re
import argparse

import gcMapExplorer.lib as gmlib

from gcMapExplorer.config import getConfig

# get configuration
config = getConfig()

description = \
"""Import Bin-Contact format files
=================================================
In this format, two separate files are available. One file contains bins
information and other contains contact frequency.

These types of files are present in following GEO data:
* http://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE61471
* http://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE34453

This format contains a pair of file:
BIN file:
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
cbin	chr	from.coord	to.coord	count
1	2L	0	160000	747
2	2L	160000	320000	893
3	2L	320000	480000	1056
4	2L	480000	640000	1060
5	2L	640000	800000	978
6	2L	800000	960000	926
.
.
.
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

CONTACT file in list format:
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
cbin1	cbin2	expected_count	observed_count
1	1	40.245201	21339
1	2	83.747499	5661
1	3	92.12501	1546
1	4	93.401273	864
1	5	87.265472	442
.
.
.
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


Both BIN and CONTACT files are neccessary for the conversion.

=================================================
"""

inputBinFileHelp = \
""" Input BIN file as shown above.

"""

inputContactFileHelp = \
""" Input CONTACT file as shown above.

"""

ccmapSuffixHelp = \
""" Use this to convert all contact maps to ccmaps file. Provide suffix of
ccmap file names with this option and it will enable the conversion.

Ouput ccmap file name is generated outmatically as follows;
<chromosome>_<resolution>_<suffix>.ccmap

Note that -od/--out-dir option is also required because all ccmaps will be
saved in this directory.

"""

outDirHelp = \
""" Directory to save all ccmap files. It should be provided when -ccmap/--ccmap
option is used.

"""

fileGCMapHelp = \
"""Provide gcmap file to convert all contact maps into one gcmap file.
File name should contain full path because -od/--out-dir is not considered
for thi conversion.

"""

downsampleMethodHelp = \
"""Downsampling method to coarsen the resolution in gcmap file. The option is
intended to use with -gcm/--gcmap option. Three accepted methods are
        sum  : sum of values,
        mean : Average of values and
        max  : Maximum of values.

This option generates all coarser maps where resolutions will be coarsened by
a factor of two, consequetively. e.g.: In case of 10 kb input resolution,
downsampled maps of "20kb", "40kb", "80kb", "160kb", "320kb" etc. will be
generated until, map size is less than 500.

"""

def main():

    # Construct command line arguments and parsed it
    parser, args = parseArguments()

    # Input Bin file
    inputBinFile = None
    if args.inputBinFile is not None:
        inputBinFile = args.inputBinFile.name
        args.inputBinFile.close()
    else:
        showErrorAndExit(parser, "No BIN Input File!!!\n")

    # Input contact file
    inputContactFile = None
    if args.inputContactFile is not None:
        inputContactFile = args.inputContactFile.name
        args.inputContactFile.close()
    else:
        showErrorAndExit(parser, "No CONTACT Input File!!!\n")

    if args.ccmapSuffix is not None and args.outDir is None:
        msg = "No output directory is given for ccmap files!!!\n"
        showErrorAndExit(parser, msg)

    if args.ccmapSuffix is None and args.fileGCMap is None:
        showErrorAndExit(parser, "No output format directed!!!\n")

    # Check for scratch directory
    if not os.path.isdir(args.workDir):
        showErrorAndExit(parser, '\nScratch Dirctory "{0}" not found !!!\n'.format(args.workDir))

    # Initialize
    binContactReader = gmlib.importer.BinsNContactFilesHandler(
                                        inputBinFile,
                                        inputContactFile,
                                        workDir=args.workDir)

    # Convert ccmaps
    if args.ccmapSuffix is not None:
        # Save ccmaps
        binContactReader.save_ccmaps(args.outDir, args.ccmapSuffix)

    # Convert gcmap
    if args.fileGCMap is not None:
        binContactReader.save_gcmap(args.fileGCMap,
                                    coarsingMethod=args.coarsingMethod,
                                    compression=args.compression)

def parseArguments():
    parser = argparse.ArgumentParser(
                prog='gcMapExplorer bc2cmap',
                description=description,
                formatter_class=argparse.RawTextHelpFormatter)

    parser.add_argument('-ib', '--input-bin', action='store',
                        type=argparse.FileType('r'),
                        metavar='nm_none_160000.bins',
                        dest='inputBinFile', help=inputBinFileHelp)

    parser.add_argument('-ic', '--input-contact', action='store',
                        type=argparse.FileType('r'),
                        metavar='nm_none_160000.n_contact',
                        dest='inputContactFile', help=inputContactFileHelp)

    parser.add_argument('-ccm', '--ccmap', action='store', dest='ccmapSuffix',
                        metavar='RawObserved', help=ccmapSuffixHelp)

    parser.add_argument('-od', '--out-dir', action='store', dest='outDir',
                        help='Directory where all ccmap files will be saved.\n')

    parser.add_argument('-gcm', '--gcmap', action='store', dest='fileGCMap',
                        metavar='inOut.gcmap', help=fileGCMapHelp)

    parser.add_argument('-cmeth', '--compression-method', action='store',
                        dest='compression', metavar='lzf',
                        choices=['lzf', 'gzip'], default='lzf',
                        help='Data compression method for gcmap file.\n')

    parser.add_argument('-dmeth', '--downsample-method', action='store',
                        dest='coarsingMethod', metavar='sum',
                        choices=['max', 'mean', 'sum'], default='sum',
                        help=downsampleMethodHelp)

    parser.add_argument('-wd', '--work-dir', action='store', dest='workDir',
                        default=config['Dirs']['WorkingDirectory'],
                        metavar=config['Dirs']['WorkingDirectory'],
                        help='Directory where temporary files will be stored.')

    idx = sys.argv.index("bc2cmap")+1
    args = parser.parse_args(args=sys.argv[idx:])

    return parser, args


def checkFileExist(filename, parser):
    if not os.path.isfile(filename):
        parser.print_help()
        print("\n===== ERROR =======")
        print("File: {0} does not exist !!!\n".format(filename))
        print("See Usage Above!!!")
        sys.exit(-1)


def showErrorAndExit(parser, message):
    parser.print_help()
    print("\n===== ERROR =======")
    print(message)
    print("See Usage Above!!!")
    sys.exit(-1)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
#
# Author: Rajendra Kumar
#
# This file is part of gcMapExplorer
# Copyright (C) 2016-2017  Rajendra Kumar, Ludvig Lizana, Per Stenberg
#
# gcMapExplorer is a free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gcMapExplorer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gcMapExplorer.  If not, see <http://www.gnu.org/licenses/>.
#
#=============================================================================

import os
import sys
import re
import argparse

import gcMapExplorer.lib as gmlib

from gcMapExplorer.config import getConfig

# get configuration
config = getConfig()

description = \
"""Normalization of contact map using Knight-Ruiz matrix balancing
===============================================================

The Knight-Ruiz (KR) matrix balancing is a fast algorithm to normalize a
symmetric matrix. A doubly stochastic matrix is obatined after this
normalization. In this matrix, sum of rows and columns are equal to one.

For more details, see publication: http://dx.doi.org/10.1093/imanum/drs019

Please cite:
P.A. Knight and D. Ruiz (2013). A fast algorithm for matrix balancing (2013).
IMA Journal of Numerical Analysis, 33, 1029-1047.
http://dx.doi.org/10.1093/imanum/drs019

================================================================
"""

inputFileHelp = \
""" Input ccmap or gcmap file.

"""

inputFileFormatHelp = \
""" Input format: 'ccmap' or 'gcmap'.

"""


outputFileHelp = \
""" Output ccmap or gcmap file.

When input file is ccmap, ouput file can be gcmap. However, when a input file
is gcmap, output file will be only in gcmap.

"""

outputFileFormatHelp = \
""" Output format: 'ccmap' or 'gcmap'.

When input file is ccmap, ouput file can be gcmap. However, when a input file
is gcmap, output file will be only in gcmap.

"""

memoryHelp = \
""" The memory used for calculation. Acceptable keywords are 'RAM' or 'HDD'.

In case of RAM, memory is used for the calculation. In case of Disk, all
intermediate steps will use DIsk Drive to store intermediate data.

This option is ONLY VALID when input file is in ccmap format.

"""

vminHelp = \
""" Minimum thershold value for normalization.
If contact frequency is less than or equal to this thershold value,
this value is discarded during normalization.

"""

vmaxHelp = \
""" Maximum thershold value for normalization.
If contact frequency is greater than or equal to this thershold value,
this value is discarded during normalization.

"""

mapSizeCeilingForMemoryHelp = \
""" Maximum size of contact map allowed for calculation using RAM.
If map size or shape is larger than this value, normalization will be
performed using disk (HDD). This option is ONLY VALID when input file is in
gcmap format.

"""
tolHelp = \
""" Tolerance for matrix balancing.
Smaller tolreance increases accuracy in sums of rows and columns.

"""

percentile_thershold_no_data_help = \
""" It can be used to filter the map, where rows/columns with largest numbers
of missing data can be discarded. Its value should be between 1 and 100.
This options discard the rows and columns which are above this percentile.
For example: if this value is 99, those rows or columns will be discarded which
contains larger than number of zeros (missing data) at 99 percentile.

To calculate percentile, all blank rows are removed, then in all rows, number
of zeros are counted. Afterwards, number of zeros at input percentile is
obtained. In next step, if a row contain number of zeros larger than this
percentile value, the whole row and column is assigned to have missing data.
This percentile indicates highest numbers of zeros (missing data) in given
rows/columns.

"""

thershold_data_occup_help = \
""" It can be used to filter the map, where rows/columns with largest numbers
of missing data can be discarded.This ratio is:
  (number of bins with data) / (total number of bins in the given row/column)

For example: if -tdo = 0.8, then all rows containing more than 20%% of
missing data will be discarded.

"""

citation = \
"""
Please Cite
===========
P.A. Knight and D. Ruiz (2013). A fast algorithm for matrix balancing (2013).
IMA Journal of Numerical Analysis, 33, 1029-1047.
http://dx.doi.org/10.1093/imanum/drs019

"""


def main():

    # Construct command line arguments and parsed it
    parser, args = parseArguments()

    # check input file exist
    if args.inputFile is not None:
        checkFileExist(args.inputFile, parser)
    else:
        msg = 'No input file!!!'
        showErrorAndExit(parser, msg)

    # check input file format provided
    if args.inputFileFormat is None:
        msg = 'No input file format given !!!'
        showErrorAndExit(parser, msg)

    # check output file provided
    if args.outputFile is None:
        msg = 'No output file name given !!!'
        showErrorAndExit(parser, msg)

    # check output file format provided
    if args.outputFileFormat is None:
        msg = 'No output file format given !!!'
        showErrorAndExit(parser, msg)

    if args.outputFileFormat == 'ccmap' and args.inputFileFormat == 'gcmap':
        msg = 'With gcmap input format, output format should be gcmap format.'
        showErrorAndExit(parser, msg)

    if args.percentile_thershold_no_data is not None and args.thershold_data_occup is not None:
        msg = 'Both Percentile and Fraction filters cannot be used simultaneously!!'
        showErrorAndExit(parser, msg)

    if args.inputFileFormat == 'gcmap' and args.mapSizeCeilingForMemory is None:
        msg = 'Provide Map Size Celing value for memory! \n See -mscm/--map-size-ceiling-for-memory option.'
        showErrorAndExit(parser, msg)

    if args.outputFileFormat == 'ccmap' and args.inputFileFormat == 'ccmap':
        gmlib.normalizer.normalizeCCMapByKR(args.inputFile, memory=args.memory, tol=args.tol,
                                            vmin=args.vmin, vmax=args.vmax, outFile=args.outputFile,
                                            percentile_thershold_no_data=args.percentile_thershold_no_data,
                                            thershold_data_occup=args.thershold_data_occup,
                                            workDir=args.workDir)

    if args.outputFileFormat == 'gcmap' and args.inputFileFormat == 'ccmap':
        norm_ccmap = gmlib.normalizer.normalizeCCMapByKR(args.inputFile, memory=args.memory, tol=args.tol,
                                            vmin=args.vmin, vmax=args.vmax,
                                            percentile_thershold_no_data=args.percentile_thershold_no_data,
                                            thershold_data_occup=args.thershold_data_occup,
                                            workDir=args.workDir)
        gmlib.gcmap.addCCMap2GCMap(norm_ccmap, args.outputFile, compression=args.compression)


    if args.outputFileFormat == 'gcmap' and args.inputFileFormat == 'gcmap':
        gmlib.normalizer.normalizeGCMapByKR(args.inputFile, args.outputFile,
                                            mapSizeCeilingForMemory=args.mapSizeCeilingForMemory,
                                            tol=args.tol, vmin=args.vmin, vmax=args.vmax,
                                            percentile_thershold_no_data=args.percentile_thershold_no_data,
                                            thershold_data_occup=args.thershold_data_occup,
                                            workDir=args.workDir)

    print(citation)

def parseArguments():
    parser = argparse.ArgumentParser(
                prog='gcMapExplorer normKR',
                description=description,
                formatter_class=argparse.RawTextHelpFormatter)

    parser.add_argument('-i', '--input', action='store',
                        metavar='input.gcmap',
                        dest='inputFile', help=inputFileHelp)

    parser.add_argument('-fi', '--format-input', action='store',
                        metavar='gcmap',
                        choices=['gcmap', 'ccmap', 'hicmap'],
                        dest='inputFileFormat', help=inputFileFormatHelp)

    parser.add_argument('-o', '--output', action='store',
                        metavar='output.gcmap',
                        dest='outputFile', help=outputFileHelp)

    parser.add_argument('-fo', '--format-output', action='store',
                        metavar='gcmap',
                        choices=['gcmap', 'ccmap'],
                        dest='outputFileFormat', help=outputFileFormatHelp)

    parser.add_argument('-t', '--tolerance', action='store',
                        dest='tol', metavar=1e-12,
                        type = float, default=1e-12,
                        help=tolHelp)

    parser.add_argument('-m', '--memory', action='store',
                        dest='memory', metavar='RAM',
                        choices=['RAM', 'HDD'], default='RAM',
                        help=memoryHelp)

    parser.add_argument('-vmax', '--maximum-value', action='store',
                        dest='vmax', type=float, metavar=None,
                        help=vminHelp)

    parser.add_argument('-vmin', '--minimum-value', action='store',
                        dest='vmin', metavar=None, type=float,
                        help=vmaxHelp)

    parser.add_argument('-mscm', '--map-size-ceiling-for-memory', action='store',
                        dest='mapSizeCeilingForMemory', metavar='20000',
                        type = int,
                        help=mapSizeCeilingForMemoryHelp)

    parser.add_argument('-ptnd', '--percentile-thershold-no-data', action='store',
                        dest='percentile_thershold_no_data', metavar=99,
                        type = float,
                        help=percentile_thershold_no_data_help)

    parser.add_argument('-tdo', '--thershold-data-occupancy', action='store',
                        dest='thershold_data_occup', metavar=0.8,
                        type = float,
                        help=thershold_data_occup_help)

    parser.add_argument('-cmeth', '--compression-method', action='store',
                        dest='compression', metavar='lzf',
                        choices=['lzf', 'gzip'], default='lzf',
                        help='Data compression method for output gcmap file.\n')

    parser.add_argument('-wd', '--work-dir', action='store', dest='workDir',
                        default=config['Dirs']['WorkingDirectory'],
                        metavar=config['Dirs']['WorkingDirectory'],
                        help='Directory where temporary files will be stored.')

    idx = sys.argv.index("normKR")+1
    args = parser.parse_args(args=sys.argv[idx:])

    return parser, args


def checkFileExist(filename, parser):
    if not os.path.isfile(filename):
        parser.print_help()
        print("\n===== ERROR =======")
        print("File: {0} does not exist !!!\n".format(filename))
        print("See Usage Above!!!")
        sys.exit(-1)


def showErrorAndExit(parser, message):
    parser.print_help()
    print("\n===== ERROR =======")
    print(message)
    print("See Usage Above!!!")
    sys.exit(-1)
